/*
	CreateSpatialTemplateCmd.cc - make spatial parameter template files for wave

	Will 18, 2001: will@opaline.demon.co.uk
*/


#include <DMenuCmd.h>
#include "OpResources.h"
#include "CreateSpatialTemplateCmd.h"
#include "CreateSpatialScoreCmd.h"
#include "AppWindow.h"
#include <MessageDialogManager.h>
#include <GraphIntCmd.h>
#include <GraphFltCmd.h>
#include "FileAccess.h"
#include <LineEdit.h>
#include "MixwaveUtils.h"
#include "RunCsoundCmd.h"
#include "MixwaveUtils.h"
#include "ShapeSpatialPakTemplateCmd.h"
#include "RawToWavCmd.h"


CreateSpatialTemplateCmd::CreateSpatialTemplateCmd ( const char *name, int active, unsigned int waveNum,
							SpatialPak *spatialPak,
							CreateSpatialTemplateCallback callback,
							void *clientData ) : NoUndoCmd ( name, active )
{
	_name = name;
	_waveNum = waveNum;
	_spatialPak = spatialPak;
	_callback = callback;
	_clientData = clientData;
	_currentSamples = 0;
	_min = 0;
	_max = 0;
	_viewRealised = False;
	_menuRealised = False;
	_pathMenuRealised = False;
	_rvbViewRealised = False;
	_rangeViewRealised = False;
	_bwViewRealised = False;
	_sweepViewRealised = False;
	_panViewRealised = False;
	_dopplerViewRealised = False;
	_rvbLViewRealised = False;
	_rvbRViewRealised = False;
	_panLViewRealised = False;
	_panRViewRealised = False;
	_bwLViewRealised = False;
	_bwRViewRealised = False;
	_sweepLViewRealised = False;
	_sweepRViewRealised = False;
	_ampLViewRealised = False;
	_ampRViewRealised = False;
	_panTypeEditRealised = False;
	_panTypeEdit = NULL;
	_filterTypeEditRealised = False;
	_filterTypeEdit = NULL;
	_angularSepEditRealised = False;
	_angularSepEdit = NULL;
	_signalScaleEditRealised = False;
	_signalScaleEdit = NULL;
	_detuneEditRealised = False;
	_detuneEdit = NULL;
	_phaserEditRealised = False;
	_phaserEdit = NULL;
	_rvbTailEditRealised = False;
	_rvbTailEdit = NULL;
	_panLTypeEditRealised = False;
	_panLTypeEdit = NULL;
	_panRTypeEditRealised = False;
	_panRTypeEdit = NULL;
	_delayLEditRealised = False;
	_delayLEdit = NULL;
	_delayREditRealised = False;
	_delayREdit = NULL;
	_filterLTypeEditRealised = False;
	_filterLTypeEdit = NULL;
	_filterRTypeEditRealised = False;
	_filterRTypeEdit = NULL;
	_filterRTypeEditRealised = False;
	_filterRTypeEdit = NULL;
	_rvbRangeEditRealised = False;
	_rvbRangeEdit = NULL;
	_distRangeEditRealised = False;
	_distRangeEdit = NULL;
	_bwRangeEditRealised = False;
	_bwRangeEdit = NULL;
	_sweepRangeEditRealised = False;
	_sweepRangeEdit = NULL;
	_dopplerRangeEditRealised = False;
	_dopplerRangeEdit = NULL;
	_rvblRangeEditRealised = False;
	_rvblRangeEdit = NULL;
	_rvbrRangeEditRealised = False;
	_rvbrRangeEdit = NULL;
	_bwlRangeEditRealised = False;
	_bwlRangeEdit = NULL;
	_bwrRangeEditRealised = False;
	_bwrRangeEdit = NULL;
	_sweeplRangeEditRealised = False;
	_sweeplRangeEdit = NULL;
	_sweeprRangeEditRealised = False;
	_sweeprRangeEdit = NULL;
	_amplRangeEditRealised = False;
	_amplRangeEdit = NULL;
	_amprRangeEditRealised = False;
	_amprRangeEdit = NULL;
	_cssCmd = NULL;
	_runCsndCmd = NULL;
	_shapeCmd = NULL;
	_shapeCmdRealised = False;
	_error = 0;
}


CreateSpatialTemplateCmd::~CreateSpatialTemplateCmd()
{
	exit();
}


void CreateSpatialTemplateCmd::doCommand()
{
	if ( _viewRealised == True )
		return;

	MixwaveUtils *utils = new MixwaveUtils;
	char viewName[256];

	if ( (*wavePak)[_waveNum-1].graphNum != 0 );
	{
		if ( (*wavePak)[_waveNum-1].waveDataCmd != NULL )
			(*wavePak)[_waveNum-1].graphCmd->close();
		
		if ( (*wavePak)[_waveNum-1].graphCmd != NULL )
			(*wavePak)[_waveNum-1].graphCmd->close();
	}

	_bytesPerSample = (*wavePak)[_waveNum-1].pWave->bitsPerSample() / 8;
	// create view objects for wave
	(*wavePak)[_waveNum-1].pWave->filePos = (*wavePak)[_waveNum-1].dataOffset<<1;
	(*wavePak)[_waveNum-1].fLock = ACTIVE;
	(*wavePak)[_waveNum-1].pWave->LoadBlock ((*wavePak)[_waveNum-1].pWave->filePos,(*wavePak)[_waveNum-1].intArraySize<<1);
	(*wavePak)[_waveNum-1].fLock = INACTIVE;

	createWaveDataArray ( _waveNum, (*wavePak)[_waveNum-1].intArraySize<<1 );

	GraphIntCmd *graphCmd = new GraphIntCmd ( (*wavePak)[_waveNum-1].pWave->name(),
						TRUE, &CreateSpatialTemplateCmd::waveViewCallback,
						mixwaveResources->numDigits(),
						(*wavePak)[_waveNum-1].pIntArray,
						(*wavePak)[_waveNum-1].dataOffset,
						_currentSamples, (*wavePak)[_waveNum-1].pWave->totalSamples(),
						(*wavePak)[_waveNum-1].pWave->waveNum(),
						( void * ) this );

	long waveSamples = _currentSamples;
	(*wavePak)[_waveNum-1].graphCmd = graphCmd;
	// save the current view ID for the wave
	(*wavePak)[_waveNum-1].graphNum = (*wavePak)[_waveNum-1].pWave->waveNum();
	graphCmd->setDatumValueBounds ( mixwaveResources->minY(), mixwaveResources->maxY() );

	if ( mixwaveResources->minY() < 0 )	// set by WaveDefaultsCmd bitsPerSample
		graphCmd->centreZero ( True );	// set to signed data type
	else
		graphCmd->centreZero ( False );

	// get the user-defined window size string into ints
	int width, height;
	sscanf ( mixwaveResources->viewWindowSize(), "%dx%d", &width, &height );
	graphCmd->setWindowSize ( width, height );
	graphCmd->execute();
	graphCmd->setSelection ( (*wavePak)[_waveNum-1].startSel, (*wavePak)[_waveNum-1].endSel );

	WaveDataCmd *waveDataCmd = new WaveDataCmd ( (*wavePak)[_waveNum-1].pWave->name(),
							TRUE, &CreateSpatialTemplateCmd::waveDataCmdCallback,
							(*wavePak)[_waveNum-1].pWave,
							(*wavePak)[_waveNum-1].pIntArray,
							mixwaveResources->numDigits(),
							(*wavePak)[_waveNum-1].dataOffset,
							_currentSamples,
							( void * ) this );

	// save the current wave data object pointer
	(*wavePak)[_waveNum-1].waveDataCmd = waveDataCmd;
	waveDataCmd->setDatumValueBounds ( mixwaveResources->minY(), mixwaveResources->maxY() );
	waveDataCmd->setFPDigits ( mixwaveResources->fpDigits() );

	waveDataCmd->execute();

	_menu = new DMenuCmd ( ( char * ) _name, TRUE, FALSE,
				&CreateSpatialTemplateCmd::menuSelectionCallback,
				( void * ) this, "Sound source parameters",
				"spatialise wave",
				"reverb envelope", "distance envelope",
				"bandwidth envelope", "sweep freq. envelope",
				"pan envelope", "doppler pitch envelope",
				"pan function",
				"filter type", "angular separation",
				"scale factor", "detune value", "phaser status",
				"reverb tail", "Quit", NULL );

	_menuRealised = True;
	_menu->execute();

	_pathMenu = new DMenuCmd ( ( char * ) _name, TRUE, FALSE,
				&CreateSpatialTemplateCmd::pathMenuSelectionCallback,
				( void * ) this, "Reflection parameters",
				"L rvb level envelope", "R rvb level envelope",
				"L pan envelope", "R pan envelope",
				"L pan function", "R pan function",
				"L bw envelope", "R bw envelope",
				"L sweep envelope", "R sweep envelope",
				"L amplitude envelope", "R amplitude envelope",
				"L delay", "R delay",
				"L filter type", "R filter type",
				"Quit", NULL );

	_pathMenuRealised = True;
	_pathMenu->execute();

	_paramRangeMenu = new DMenuCmd ( ( char * ) _name, TRUE, FALSE,
					&CreateSpatialTemplateCmd::paramRangeMenuSelectionCallback,
					( void * ) this, "Range parameters",
					"Reverb", "Distance", "Bandwidth",
					"Sweep freq.", "Doppler pitch", "L reverb",
					"R reverb", "L bandwidth", "R bandwidth",
					"L sweep freq.", "R sweep freq.",
					"L amplitude", "R amplitude", "Quit", NULL );
									
	_paramRangeMenuRealised = True;
	_paramRangeMenu->execute();

	// view objects for dependent templates
	unsigned int viewNumber = 1;

	strncpy ( _rvbFileName, _spatialPak->name, sizeof ( _rvbFileName ) );
	_rvbFileName [ sizeof ( _rvbFileName ) - 1 ] = 0;
	strncat ( _rvbFileName, "_reverb.flt", sizeof ( _rvbFileName ) - strlen ( _rvbFileName ) - 1 );	// s/b a resources-defined suffix
	_rvbFileName [ sizeof ( _rvbFileName ) - 1 ] = 0;

	FileAccess *rvbIn = new FileAccess ( _rvbFileName, "r+" );

	if ( rvbIn->errNum() != 0  )
		initError ( _rvbFileName, rvbIn->error() );
	else
	{
		_spatialPak->rvbDataSize = ( long ) ( float ) rvbIn->fileSize() / sizeof ( float );
		
		if ( (*wavePak)[_waveNum-1].pWave->totalSamples() == 0 )
			_rvbScale = 1;
		else
			_rvbScale = ( float ) _spatialPak->rvbDataSize / (*wavePak)[_waveNum-1].pWave->totalSamples();

		_spatialPak->rvbArraySize = ( long ) ( ( ( float ) waveSamples * _rvbScale ) + 0.5 );

		if ( _spatialPak->rvbArraySize > _spatialPak->rvbDataSize )
			_spatialPak->rvbArraySize = _spatialPak->rvbDataSize;

		_spatialPak->rvbFilePos = _spatialPak->rvbDataOffset * sizeof ( float );

		_spatialPak->rvbArray = createFloatDataArray ( _rvbFileName, _spatialPak->rvbArray, _spatialPak->rvbArraySize * sizeof ( float ),
									 					_spatialPak->rvbFilePos );

		snprintf ( viewName, sizeof ( viewName ), "Reverb send %s", utils->fileFromPath ( _rvbFileName ) );
		viewName [ sizeof ( viewName ) - 1 ] = 0;

		_spatialPak->rvbView = new GraphFltCmd ( viewName,
							TRUE, &CreateSpatialTemplateCmd::viewReverbTemplateCmdCallback,
							mixwaveResources->numDigits(),
							_spatialPak->rvbArray,
							_spatialPak->rvbDataOffset,
							_currentSamples,
							_spatialPak->rvbDataSize,
							viewNumber,
							( void * ) this );

		_spatialPak->rvbGraphNum = viewNumber;
		_spatialPak->rvbView->setWindowSize ( width, height );
		_spatialPak->rvbView->autoClose ( False );	// wave window closes view
		viewNumber++;
	}
	
	delete rvbIn;

	strncpy ( _rangeFileName, _spatialPak->name, sizeof ( _rangeFileName ) );
	_rangeFileName [ sizeof ( _rangeFileName ) - 1 ] = 0;
	strncat ( _rangeFileName, "_range.flt", sizeof ( _rangeFileName ) - strlen ( _rangeFileName ) - 1 );	// s/b a resources-defined suffix
	_rangeFileName [ sizeof ( _rangeFileName ) - 1 ] = 0;

	FileAccess *rangeIn = new FileAccess ( _rangeFileName, "r+" );

	if ( rangeIn->errNum() != 0  )
		initError ( _rangeFileName, rangeIn->error() );
	else
	{
		_spatialPak->rangeDataSize = ( long ) ( float ) rangeIn->fileSize() / sizeof ( float );
		
		if ( (*wavePak)[_waveNum-1].pWave->totalSamples() == 0 )
			_rangeScale = 1;
		else
			_rangeScale = ( float ) _spatialPak->rangeDataSize / (*wavePak)[_waveNum-1].pWave->totalSamples();

		_spatialPak->rangeArraySize = ( long ) ( ( ( float ) waveSamples * _rangeScale ) + 0.5 );

		if ( _spatialPak->rangeArraySize > _spatialPak->rangeDataSize )
			_spatialPak->rangeArraySize = _spatialPak->rangeDataSize;

		_spatialPak->rangeFilePos = _spatialPak->rangeDataOffset * sizeof ( float );
		_spatialPak->rangeArray = createFloatDataArray ( _rangeFileName, _spatialPak->rangeArray, _spatialPak->rangeArraySize * sizeof ( float ),
									 						_spatialPak->rangeFilePos );

		snprintf ( viewName, sizeof ( viewName ), "Range %s", utils->fileFromPath ( _rangeFileName ) );
		viewName [ sizeof ( viewName ) - 1 ] = 0;

		_spatialPak->rangeView = new GraphFltCmd ( viewName,
							TRUE, &CreateSpatialTemplateCmd::viewRangeTemplateCmdCallback,
							mixwaveResources->numDigits(),
							_spatialPak->rangeArray,
							_spatialPak->rangeDataOffset,
							_currentSamples,
							_spatialPak->rangeDataSize,
							viewNumber,
							( void * ) this );

		_spatialPak->rangeGraphNum = viewNumber;
		_spatialPak->rangeView->setWindowSize ( width, height );
		_spatialPak->rangeView->autoClose ( False );	// wave window closes view
		viewNumber++;
	}
	
	delete rangeIn;

	strncpy ( _bwFileName, _spatialPak->name, sizeof ( _bwFileName ) );
	_bwFileName [ sizeof ( _bwFileName ) - 1 ] = 0;
	strncat ( _bwFileName, "_bwidth.flt", sizeof ( _bwFileName ) - strlen ( _bwFileName ) - 1 );	// s/b a resources-defined suffix
	_bwFileName [ sizeof ( _bwFileName ) - 1 ] = 0;

	FileAccess *bwIn = new FileAccess ( _bwFileName, "r+" );

	if ( bwIn->errNum() != 0  )
		initError ( _bwFileName, bwIn->error() );
	else
	{
		_spatialPak->bwDataSize = ( long ) ( float ) bwIn->fileSize() / sizeof ( float );
		
		if ( (*wavePak)[_waveNum-1].pWave->totalSamples() == 0 )
			_bwScale = 1;
		else
			_bwScale = ( float ) _spatialPak->bwDataSize / (*wavePak)[_waveNum-1].pWave->totalSamples();

		_spatialPak->bwArraySize = ( long ) ( ( ( float ) waveSamples * _bwScale ) + 0.5 );

		if ( _spatialPak->bwArraySize > _spatialPak->bwDataSize )
			_spatialPak->bwArraySize = _spatialPak->bwDataSize;

		_spatialPak->bwFilePos = _spatialPak->bwDataOffset * sizeof ( float );
		_spatialPak->bwArray = createFloatDataArray ( _bwFileName, _spatialPak->bwArray, _spatialPak->bwArraySize * sizeof ( float ),
									 					_spatialPak->bwFilePos );

		snprintf ( viewName, sizeof ( viewName ), "Bandwidth (Hz) %s", utils->fileFromPath ( _bwFileName ) );
		viewName [ sizeof ( viewName ) - 1 ] = 0;

		_spatialPak->bwView = new GraphFltCmd ( viewName,
							TRUE, &CreateSpatialTemplateCmd::viewBwidthTemplateCmdCallback,
							mixwaveResources->numDigits(),
							_spatialPak->bwArray,
							_spatialPak->bwDataOffset,
							_currentSamples,
							_spatialPak->bwDataSize,
							viewNumber,
							( void * ) this );

		_spatialPak->bwGraphNum = viewNumber;
		_spatialPak->bwView->setWindowSize ( width, height );
		_spatialPak->bwView->autoClose ( False );	// wave window closes view
		viewNumber++;
	}
	
	delete bwIn;

	strncpy ( _sweepFileName, _spatialPak->name, sizeof ( _sweepFileName ) );
	_sweepFileName [ sizeof ( _sweepFileName ) - 1 ] = 0;
	strncat ( _sweepFileName, "_sweep.flt", sizeof ( _sweepFileName ) - strlen ( _sweepFileName ) - 1 );	// s/b a resources-defined suffix
	_sweepFileName [ sizeof ( _sweepFileName ) - 1 ] = 0;

	FileAccess *sweepIn = new FileAccess ( _sweepFileName, "r+" );

	if ( sweepIn->errNum() != 0  )
		initError ( _sweepFileName, sweepIn->error() );
	else
	{
		_spatialPak->sweepDataSize = ( long ) ( float ) sweepIn->fileSize() / sizeof ( float );
		
		if ( (*wavePak)[_waveNum-1].pWave->totalSamples() == 0 )
			_sweepScale = 1;
		else
			_sweepScale = ( float ) _spatialPak->sweepDataSize / (*wavePak)[_waveNum-1].pWave->totalSamples();

		_spatialPak->sweepArraySize = ( long ) ( ( ( float ) waveSamples * _sweepScale ) + 0.5 );

		if ( _spatialPak->sweepArraySize > _spatialPak->sweepDataSize )
			_spatialPak->sweepArraySize = _spatialPak->sweepDataSize;

		_spatialPak->sweepFilePos = _spatialPak->sweepDataOffset * sizeof ( float );
		_spatialPak->sweepArray = createFloatDataArray ( _sweepFileName, _spatialPak->sweepArray, _spatialPak->sweepArraySize * sizeof ( float ),
									 					_spatialPak->sweepFilePos );

		snprintf ( viewName, sizeof ( viewName ), "Sweep Freq. (Hz) %s", utils->fileFromPath ( _sweepFileName ) );
		viewName [ sizeof ( viewName ) - 1 ] = 0;

		_spatialPak->sweepView = new GraphFltCmd ( viewName,
							TRUE, &CreateSpatialTemplateCmd::viewSweepTemplateCmdCallback,
							mixwaveResources->numDigits(),
							_spatialPak->sweepArray,
							_spatialPak->sweepDataOffset,
							_currentSamples,
							_spatialPak->sweepDataSize,
							viewNumber,
							( void * ) this );

		_spatialPak->sweepGraphNum = viewNumber;
		_spatialPak->sweepView->setWindowSize ( width, height );
		_spatialPak->sweepView->autoClose ( False );	// wave window closes view
		viewNumber++;
	}
	
	delete sweepIn;

	strncpy ( _panFileName, _spatialPak->name, sizeof ( _panFileName ) );
	_panFileName [ sizeof ( _panFileName ) - 1 ] = 0;
	strncat ( _panFileName, "_pan.flt", sizeof ( _panFileName ) - strlen ( _panFileName ) - 1 );	// s/b a resources-defined suffix
	_panFileName [ sizeof ( _panFileName ) - 1 ] = 0;

	FileAccess *panIn = new FileAccess ( _panFileName, "r+" );

	if ( panIn->errNum() != 0  )
		initError ( _panFileName, panIn->error() );
	else
	{
		_spatialPak->panDataSize = ( long ) ( float ) panIn->fileSize() / sizeof ( float );
		
		if ( (*wavePak)[_waveNum-1].pWave->totalSamples() == 0 )
			_panScale = 1;
		else
			_panScale = ( float ) _spatialPak->panDataSize / (*wavePak)[_waveNum-1].pWave->totalSamples();

		_spatialPak->panArraySize = ( long ) ( ( ( float ) waveSamples * _panScale ) + 0.5 );

		if ( _spatialPak->panArraySize > _spatialPak->panDataSize )
			_spatialPak->panArraySize = _spatialPak->panDataSize;

		_spatialPak->panFilePos = _spatialPak->panDataOffset * sizeof ( float );
		_spatialPak->panArray = createFloatDataArray ( _panFileName, _spatialPak->panArray, _spatialPak->panArraySize * sizeof ( float ),
									 					_spatialPak->panFilePos );

		snprintf ( viewName, sizeof ( viewName ), "Pan %s", utils->fileFromPath ( _panFileName ) );
		viewName [ sizeof ( viewName ) - 1 ] = 0;

		_spatialPak->panView = new GraphFltCmd ( viewName,
							TRUE, &CreateSpatialTemplateCmd::viewPanTemplateCmdCallback,
							mixwaveResources->numDigits(),
							_spatialPak->panArray,
							_spatialPak->panDataOffset,
							_currentSamples,
							_spatialPak->panDataSize,
							viewNumber,
							( void * ) this );

		_spatialPak->panGraphNum = viewNumber;
		_spatialPak->panView->setWindowSize ( width, height );
		_spatialPak->panView->autoClose ( False );	// wave window closes view
		viewNumber++;
	}
	
	delete panIn;

	strncpy ( _dplrFileName, _spatialPak->name, sizeof ( _dplrFileName ) );
	_dplrFileName [ sizeof ( _dplrFileName ) - 1 ] = 0;
	strncat ( _dplrFileName, "_dplr.flt", sizeof ( _dplrFileName ) - strlen ( _dplrFileName ) - 1 );	// s/b a resources-defined suffix
	_dplrFileName [ sizeof ( _dplrFileName ) - 1 ] = 0;

	FileAccess *dopplerIn = new FileAccess ( _dplrFileName, "r+" );

	if ( dopplerIn->errNum() != 0  )
		initError ( _dplrFileName, dopplerIn->error() );
	else
	{
		_spatialPak->dopplerDataSize = ( long ) ( float ) dopplerIn->fileSize() / sizeof ( float );
		
		if ( (*wavePak)[_waveNum-1].pWave->totalSamples() == 0 )
			_dplrScale = 1;
		else
			_dplrScale = ( float ) _spatialPak->dopplerDataSize / (*wavePak)[_waveNum-1].pWave->totalSamples();

		_spatialPak->dopplerArraySize = ( long ) ( ( ( float ) waveSamples * _dplrScale ) + 0.5 );

		if ( _spatialPak->dopplerArraySize > _spatialPak->dopplerDataSize )
			_spatialPak->dopplerArraySize = _spatialPak->dopplerDataSize;

		_spatialPak->dopplerFilePos = _spatialPak->dopplerDataOffset * sizeof ( float );
		_spatialPak->dopplerArray = createFloatDataArray ( _dplrFileName, _spatialPak->dopplerArray, _spatialPak->dopplerArraySize * sizeof ( float ),
										 					_spatialPak->dopplerFilePos );

		snprintf ( viewName, sizeof ( viewName ), "Doppler shift %s", utils->fileFromPath ( _dplrFileName ) );
		viewName [ sizeof ( viewName ) - 1 ] = 0;
		_spatialPak->dopplerView = new GraphFltCmd ( viewName,
								TRUE, &CreateSpatialTemplateCmd::viewDopplerTemplateCmdCallback,
								mixwaveResources->numDigits(),
								_spatialPak->dopplerArray,
								_spatialPak->dopplerDataOffset,
								_currentSamples,
								_spatialPak->dopplerDataSize,
								viewNumber,
								( void * ) this );

		_spatialPak->dopplerGraphNum = viewNumber;
		_spatialPak->dopplerView->setWindowSize ( width, height );
		_spatialPak->dopplerView->autoClose ( False );	// wave window closes view
		viewNumber++;
	}
	
	delete dopplerIn;

	strncpy ( _rvbLFileName, _spatialPak->name, sizeof ( _rvbLFileName ) );
	_rvbLFileName [ sizeof ( _rvbLFileName ) - 1 ] = 0;
	strncat ( _rvbLFileName, "_rvbl.flt", sizeof ( _rvbLFileName ) - strlen ( _rvbLFileName ) - 1 );	// s/b a resources-defined suffix
	_rvbLFileName [ sizeof ( _rvbLFileName ) - 1 ] = 0;

	FileAccess *rvbLIn = new FileAccess ( _rvbLFileName, "r+" );

	if ( rvbLIn->errNum() != 0  )
		initError ( _rvbLFileName, rvbLIn->error() );
	else
	{
		_spatialPak->rvbLDataSize = ( long ) ( float ) rvbLIn->fileSize() / sizeof ( float );
		
		if ( (*wavePak)[_waveNum-1].pWave->totalSamples() == 0 )
			_rvblScale = 1;
		else
			_rvblScale = ( float ) _spatialPak->rvbLDataSize / (*wavePak)[_waveNum-1].pWave->totalSamples();

		_spatialPak->rvbLArraySize = ( long ) ( ( ( float ) waveSamples * _rvblScale ) + 0.5 );

		if ( _spatialPak->rvbLArraySize > _spatialPak->rvbLDataSize )
			_spatialPak->rvbLArraySize = _spatialPak->rvbLDataSize;

		_spatialPak->rvbLFilePos = _spatialPak->rvbLDataOffset * sizeof ( float );
		_spatialPak->rvbLArray = createFloatDataArray ( _rvbLFileName, _spatialPak->rvbLArray, _spatialPak->rvbLArraySize * sizeof ( float ),
									 					_spatialPak->rvbLFilePos );

		snprintf ( viewName, sizeof ( viewName ), "Left reflection reverb %s", utils->fileFromPath ( _rvbLFileName ) );
		viewName [ sizeof ( viewName ) - 1 ] = 0;

		_spatialPak->rvbLView = new GraphFltCmd ( viewName,
								TRUE, &CreateSpatialTemplateCmd::viewReverbLTemplateCmdCallback,
								mixwaveResources->numDigits(),
								_spatialPak->rvbLArray,
								_spatialPak->rvbLDataOffset,
								_currentSamples,
								_spatialPak->rvbLDataSize,
								viewNumber,
								( void * ) this );

		_spatialPak->rvbLGraphNum = viewNumber;
		_spatialPak->rvbLView->setWindowSize ( width, height );
		_spatialPak->rvbLView->autoClose ( False );	// wave window closes view
		viewNumber++;
	}
	
	delete rvbLIn;

	strncpy ( _rvbRFileName, _spatialPak->name, sizeof ( _rvbRFileName ) );
	_rvbRFileName [ sizeof ( _rvbRFileName ) - 1 ] = 0;
	strncat ( _rvbRFileName, "_rvbr.flt", sizeof ( _rvbRFileName ) - strlen ( _rvbRFileName ) - 1 );	// s/b a resources-defined suffix
	_rvbRFileName [ sizeof ( _rvbRFileName ) - 1 ] = 0;

	FileAccess *rvbRIn = new FileAccess ( _rvbRFileName, "r+" );

	if ( rvbRIn->errNum() != 0  )
		initError ( _rvbRFileName, rvbRIn->error() );
	else
	{
		_spatialPak->rvbRDataSize = ( long ) ( float ) rvbRIn->fileSize() / sizeof ( float );
		
		if ( (*wavePak)[_waveNum-1].pWave->totalSamples() == 0 )
			_rvbrScale = 1;
		else
			_rvbrScale = ( float ) _spatialPak->rvbRDataSize / (*wavePak)[_waveNum-1].pWave->totalSamples();

		_spatialPak->rvbRArraySize = ( long ) ( ( ( float ) waveSamples * _rvbrScale ) + 0.5 );

		if ( _spatialPak->rvbRArraySize > _spatialPak->rvbRDataSize )
			_spatialPak->rvbRArraySize = _spatialPak->rvbRDataSize;

		_spatialPak->rvbRFilePos = _spatialPak->rvbRDataOffset * sizeof ( float );
		_spatialPak->rvbRArray = createFloatDataArray ( _rvbRFileName, _spatialPak->rvbRArray, _spatialPak->rvbRArraySize * sizeof ( float ),
									 					_spatialPak->rvbRFilePos );

		snprintf ( viewName, sizeof ( viewName ), "Right reflection reverb %s", utils->fileFromPath ( _rvbRFileName ) );
		viewName [ sizeof ( viewName ) - 1 ] = 0;

		_spatialPak->rvbRView = new GraphFltCmd ( viewName,
								TRUE, &CreateSpatialTemplateCmd::viewReverbRTemplateCmdCallback,
								mixwaveResources->numDigits(),
								_spatialPak->rvbRArray,
								_spatialPak->rvbRDataOffset,
								_currentSamples,
								_spatialPak->rvbRDataSize,
								viewNumber,
								( void * ) this );

		_spatialPak->rvbRGraphNum = viewNumber;
		_spatialPak->rvbRView->setWindowSize ( width, height );
		_spatialPak->rvbRView->autoClose ( False );	// wave window closes view
		viewNumber++;
	}
	
	delete rvbRIn;

	strncpy ( _panLFileName, _spatialPak->name, sizeof ( _panLFileName ) );
	_panLFileName [ sizeof ( _panLFileName ) - 1 ] = 0;
	strncat ( _panLFileName, "_panl.flt", sizeof ( _panLFileName ) - strlen ( _panLFileName ) - 1 );	// s/b a resources-defined suffix
	_panLFileName [ sizeof ( _panLFileName ) - 1 ] = 0;

	FileAccess *panLIn = new FileAccess ( _panLFileName, "r+" );

	if ( panLIn->errNum() != 0  )
		initError ( _panLFileName, panLIn->error() );
	else
	{
		_spatialPak->panLDataSize = ( long ) ( float ) panLIn->fileSize() / sizeof ( float );
		
		if ( (*wavePak)[_waveNum-1].pWave->totalSamples() == 0 )
			_panlScale = 1;
		else
			_panlScale = ( float ) _spatialPak->panLDataSize / (*wavePak)[_waveNum-1].pWave->totalSamples();

		_spatialPak->panLArraySize = ( long ) ( ( ( float ) waveSamples * _panlScale ) + 0.5 );

		if ( _spatialPak->panLArraySize > _spatialPak->panLDataSize )
			_spatialPak->panLArraySize = _spatialPak->panLDataSize;

		_spatialPak->panLFilePos = _spatialPak->panLDataOffset * sizeof ( float );
		_spatialPak->panLArray = createFloatDataArray ( _panLFileName, _spatialPak->panLArray, _spatialPak->panLArraySize * sizeof ( float ),
									 					_spatialPak->panLFilePos );

		snprintf ( viewName, sizeof ( viewName ), "Left reflection pan %s", utils->fileFromPath ( _panLFileName ) );
		viewName [ sizeof ( viewName ) - 1 ] = 0;

		_spatialPak->panLView = new GraphFltCmd ( viewName,
								TRUE, &CreateSpatialTemplateCmd::viewPanLTemplateCmdCallback,
								mixwaveResources->numDigits(),
								_spatialPak->panLArray,
								_spatialPak->panLDataOffset,
								_currentSamples,
								_spatialPak->panLDataSize,
								viewNumber,
								( void * ) this );

		_spatialPak->panLGraphNum = viewNumber;
		_spatialPak->panLView->setWindowSize ( width, height );
		_spatialPak->panLView->autoClose ( False );	// wave window closes view
		viewNumber++;
	}
	
	delete panLIn;

	strncpy ( _panRFileName, _spatialPak->name, sizeof ( _panRFileName ) );
	_panRFileName [ sizeof ( _panRFileName ) - 1 ] = 0;
	strncat ( _panRFileName, "_panr.flt", sizeof ( _panRFileName ) - strlen ( _panRFileName ) - 1 );	// s/b a resources-defined suffix
	_panRFileName [ sizeof ( _panRFileName ) - 1 ] = 0;

	FileAccess *panRIn = new FileAccess ( _panRFileName, "r+" );

	if ( panRIn->errNum() != 0  )
		initError ( _panRFileName, panRIn->error() );
	else
	{
		_spatialPak->panRDataSize = ( long ) ( float ) panRIn->fileSize() / sizeof ( float );
		
		if ( (*wavePak)[_waveNum-1].pWave->totalSamples() == 0 )
			_panrScale = 1;
		else
			_panrScale = ( float ) _spatialPak->panRDataSize / (*wavePak)[_waveNum-1].pWave->totalSamples();

		_spatialPak->panRArraySize = ( long ) ( ( ( float ) waveSamples * _panrScale ) + 0.5 );

		if ( _spatialPak->panRArraySize > _spatialPak->panRDataSize )
			_spatialPak->panRArraySize = _spatialPak->panRDataSize;

		_spatialPak->panRFilePos = _spatialPak->panRDataOffset * sizeof ( float );
		_spatialPak->panRArray = createFloatDataArray ( _panRFileName, _spatialPak->panRArray, _spatialPak->panRArraySize * sizeof ( float ),
									 					_spatialPak->panRFilePos );

		snprintf ( viewName, sizeof ( viewName ), "Right reflection pan %s", utils->fileFromPath ( _panRFileName ) );
		viewName [ sizeof ( viewName ) - 1 ] = 0;

		_spatialPak->panRView = new GraphFltCmd ( viewName,
								TRUE, &CreateSpatialTemplateCmd::viewBwidthLTemplateCmdCallback,													mixwaveResources->numDigits(),
								_spatialPak->panRArray,
								_spatialPak->panRDataOffset,
								_currentSamples,
								_spatialPak->panRDataSize,
								viewNumber,
								( void * ) this );

		_spatialPak->panRGraphNum = viewNumber;
		_spatialPak->panRView->setWindowSize ( width, height );
		_spatialPak->panRView->autoClose ( False );	// wave window closes view
		viewNumber++;
	}
	
	delete panRIn;

	strncpy ( _bwLFileName, _spatialPak->name, sizeof ( _bwLFileName ) );
	_bwLFileName [ sizeof ( _bwLFileName ) - 1 ] = 0;
	strncat ( _bwLFileName, "_bwl.flt", sizeof ( _bwLFileName ) - strlen ( _bwLFileName ) - 1 );	// s/b a resources-defined suffix
	_bwLFileName [ sizeof ( _bwLFileName ) - 1 ] = 0;

	FileAccess *bwLIn = new FileAccess ( _bwLFileName, "r+" );

	if ( bwLIn->errNum() != 0  )
		initError ( _bwLFileName, bwLIn->error() );
	else
	{
		_spatialPak->bwLDataSize = ( long ) ( float ) bwLIn->fileSize() / sizeof ( float );
		
		if ( (*wavePak)[_waveNum-1].pWave->totalSamples() == 0 )
			_bwlScale = 1;
		else
			_bwlScale = ( float ) _spatialPak->bwLDataSize / (*wavePak)[_waveNum-1].pWave->totalSamples();

		_spatialPak->bwLArraySize = ( long ) ( ( ( float ) waveSamples * _bwlScale ) + 0.5 );

		if ( _spatialPak->bwLArraySize > _spatialPak->bwLDataSize )
			_spatialPak->bwLArraySize = _spatialPak->bwLDataSize;

		_spatialPak->bwLFilePos = _spatialPak->bwLDataOffset * sizeof ( float );
		_spatialPak->bwLArray = createFloatDataArray ( _bwLFileName, _spatialPak->bwLArray, _spatialPak->bwLArraySize * sizeof ( float ),
									 					_spatialPak->bwLFilePos );

		snprintf ( viewName, sizeof ( viewName ), "Left reflection bandwidth %s", utils->fileFromPath ( _bwLFileName ) );
		viewName [ sizeof ( viewName ) - 1 ] = 0;

		_spatialPak->bwLView = new GraphFltCmd ( viewName,
							TRUE, &CreateSpatialTemplateCmd::viewBwidthLTemplateCmdCallback,
							mixwaveResources->numDigits(),
							_spatialPak->bwLArray,
							_spatialPak->bwLDataOffset,
							_currentSamples,
							_spatialPak->bwLDataSize,
							viewNumber,
							( void * ) this );

		_spatialPak->bwLGraphNum = viewNumber;
		_spatialPak->bwLView->setWindowSize ( width, height );
		_spatialPak->bwLView->autoClose ( False );	// wave window closes view
		viewNumber++;
	}

	delete bwLIn;

	strncpy ( _bwRFileName, _spatialPak->name, sizeof ( _bwRFileName ) );
	_bwRFileName [ sizeof ( _bwRFileName ) - 1 ] = 0;
	strncat ( _bwRFileName, "_bwr.flt", sizeof ( _bwRFileName ) - strlen ( _bwRFileName ) - 1 );	// s/b a resources-defined suffix
	_bwRFileName [ sizeof ( _bwRFileName ) - 1 ] = 0;

	FileAccess *bwRIn = new FileAccess ( _bwRFileName, "r+" );

	if ( bwRIn->errNum() != 0  )
		initError ( _bwRFileName, bwRIn->error() );
	else
	{
		_spatialPak->bwRDataSize = ( long ) ( float ) bwRIn->fileSize() / sizeof ( float );
		
		if ( (*wavePak)[_waveNum-1].pWave->totalSamples() == 0 )
			_bwrScale = 1;
		else
			_bwrScale = ( float ) _spatialPak->bwRDataSize / (*wavePak)[_waveNum-1].pWave->totalSamples();

		_spatialPak->bwRArraySize = ( long ) ( ( ( float ) waveSamples * _bwrScale ) + 0.5 );

		if ( _spatialPak->bwRArraySize > _spatialPak->bwRDataSize )
			_spatialPak->bwRArraySize = _spatialPak->bwRDataSize;

		_spatialPak->bwRFilePos = _spatialPak->bwRDataOffset * sizeof ( float );
		_spatialPak->bwRArray = createFloatDataArray ( _bwRFileName, _spatialPak->bwRArray, _spatialPak->bwRArraySize * sizeof ( float ),
									 					_spatialPak->bwRFilePos );

		snprintf ( viewName, sizeof ( viewName ), "Right reflection bandwidth %s", utils->fileFromPath ( _bwRFileName ) );
		viewName [ sizeof ( viewName ) - 1 ] = 0;

		_spatialPak->bwRView = new GraphFltCmd ( viewName,
							TRUE, &CreateSpatialTemplateCmd::viewBwidthRTemplateCmdCallback,
							mixwaveResources->numDigits(),
							_spatialPak->bwRArray,
							_spatialPak->bwRDataOffset,
							_currentSamples,
							_spatialPak->bwRDataSize,
							viewNumber,
							( void * ) this );

		_spatialPak->bwRGraphNum = viewNumber;
		_spatialPak->bwRView->setWindowSize ( width, height );
		_spatialPak->bwRView->autoClose ( False );	// wave window closes view
		viewNumber++;
	}
	
	delete bwRIn;

	strncpy ( _sweepLFileName, _spatialPak->name, sizeof ( _sweepLFileName ) );
	_sweepLFileName [ sizeof ( _sweepLFileName ) - 1 ] = 0;
	strncat ( _sweepLFileName, "_swpl.flt", sizeof ( _sweepLFileName ) - strlen ( _sweepLFileName ) - 1 );	// s/b a resources-defined suffix
	_sweepLFileName [ sizeof ( _sweepLFileName ) - 1 ] = 0;

	FileAccess *sweepLIn = new FileAccess ( _sweepLFileName, "r+" );

	if ( sweepLIn->errNum() != 0  )
		initError ( _sweepLFileName, sweepLIn->error() );
	else
	{
		_spatialPak->sweepLDataSize = ( long ) ( float ) sweepLIn->fileSize() / sizeof ( float );
		
		if ( (*wavePak)[_waveNum-1].pWave->totalSamples() == 0 )
			_sweeplScale = 1;
		else
			_sweeplScale = ( float ) _spatialPak->sweepLDataSize / (*wavePak)[_waveNum-1].pWave->totalSamples();

		_spatialPak->sweepLArraySize = ( long ) ( ( ( float ) waveSamples * _sweeplScale ) + 0.5 );

		if ( _spatialPak->sweepLArraySize > _spatialPak->sweepLDataSize )
			_spatialPak->sweepLArraySize = _spatialPak->sweepLDataSize;

		_spatialPak->sweepLFilePos = _spatialPak->sweepLDataOffset * sizeof ( float );
		_spatialPak->sweepLArray = createFloatDataArray ( _sweepLFileName, _spatialPak->sweepLArray, _spatialPak->sweepLArraySize * sizeof ( float ),
									 						_spatialPak->sweepLFilePos );

		snprintf ( viewName, sizeof ( viewName ), "Left reflection sweep Freq. %s", utils->fileFromPath ( _sweepLFileName ) );
		viewName [ sizeof ( viewName ) - 1 ] = 0;

		_spatialPak->sweepLView = new GraphFltCmd ( viewName,
							TRUE, &CreateSpatialTemplateCmd::viewSweepLTemplateCmdCallback,
							mixwaveResources->numDigits(),
							_spatialPak->sweepLArray,
							_spatialPak->sweepLDataOffset,
							_currentSamples,
							_spatialPak->sweepLDataSize,
							viewNumber,
							( void * ) this );

		_spatialPak->sweepLGraphNum = viewNumber;
		_spatialPak->sweepLView->setWindowSize ( width, height );
		_spatialPak->sweepLView->autoClose ( False );	// wave window closes view
		viewNumber++;
	}
	
	delete sweepLIn;

	strncpy ( _sweepRFileName, _spatialPak->name, sizeof ( _sweepRFileName ) );
	_sweepRFileName [ sizeof ( _sweepRFileName ) - 1 ] = 0;
	strncat ( _sweepRFileName, "_swpr.flt", sizeof ( _sweepRFileName ) - strlen ( _sweepRFileName ) - 1 );	// s/b a resources-defined suffix
	_sweepRFileName [ sizeof ( _sweepRFileName ) - 1 ] = 0;

	FileAccess *sweepRIn = new FileAccess ( _sweepRFileName, "r+" );

	if ( sweepRIn->errNum() != 0  )
		initError ( _sweepRFileName, sweepRIn->error() );
	else
	{
		_spatialPak->sweepRDataSize = ( long ) ( float ) sweepRIn->fileSize() / sizeof ( float );
		
		if ( (*wavePak)[_waveNum-1].pWave->totalSamples() == 0 )
			_sweeprScale = 1;
		else
			_sweeprScale = ( float ) _spatialPak->sweepRDataSize / (*wavePak)[_waveNum-1].pWave->totalSamples();

		_spatialPak->sweepRArraySize = ( long ) ( ( ( float ) waveSamples * _sweeprScale ) + 0.5 );

		if ( _spatialPak->sweepRArraySize > _spatialPak->sweepRDataSize )
			_spatialPak->sweepRArraySize = _spatialPak->sweepRDataSize;

		_spatialPak->sweepRFilePos = _spatialPak->sweepRDataOffset * sizeof ( float );
		_spatialPak->sweepRArray = createFloatDataArray ( _sweepRFileName, _spatialPak->sweepRArray, _spatialPak->sweepRArraySize * sizeof ( float ),
									 						_spatialPak->sweepRFilePos );

		snprintf ( viewName, sizeof ( viewName ), "Right reflection sweep freq. %s", utils->fileFromPath ( _sweepRFileName ) );
		viewName [ sizeof ( viewName ) - 1 ] = 0;

		_spatialPak->sweepRView = new GraphFltCmd ( viewName,
							TRUE, &CreateSpatialTemplateCmd::viewSweepRTemplateCmdCallback,
							mixwaveResources->numDigits(),
							_spatialPak->sweepRArray,
							_spatialPak->sweepRDataOffset,
							_currentSamples,
							_spatialPak->sweepRDataSize,
							viewNumber,
							( void * ) this );

		_spatialPak->sweepRGraphNum = viewNumber;
		_spatialPak->sweepRView->setWindowSize ( width, height );
		_spatialPak->sweepRView->autoClose ( False );	// wave window closes view
		viewNumber++;
	}
	
	delete sweepRIn;

	strncpy ( _ampLFileName, _spatialPak->name, sizeof ( _ampLFileName ) );
	_ampLFileName [ sizeof ( _ampLFileName ) - 1 ] = 0;
	strncat ( _ampLFileName, "_ampl.flt", sizeof ( _ampLFileName ) - strlen ( _ampLFileName ) - 1 );	// s/b a resources-defined suffix
	_ampLFileName [ sizeof ( _ampLFileName ) - 1 ] = 0;

	FileAccess *ampLIn = new FileAccess ( _ampLFileName, "r+" );

	if ( ampLIn->errNum() != 0  )
		initError ( _ampLFileName, ampLIn->error() );
	else
	{
		_spatialPak->ampLDataSize = ( long ) ( float ) ampLIn->fileSize() / sizeof ( float );
		
		if ( (*wavePak)[_waveNum-1].pWave->totalSamples() == 0 )
			_amplScale = 1;
		else
			_amplScale = ( float ) _spatialPak->ampLDataSize / (*wavePak)[_waveNum-1].pWave->totalSamples();

		_spatialPak->ampLArraySize = ( long ) ( ( ( float ) waveSamples * _amplScale ) + 0.5 );

		if ( _spatialPak->ampLArraySize > _spatialPak->ampLDataSize )
			_spatialPak->ampLArraySize = _spatialPak->ampLDataSize;

		_spatialPak->ampLFilePos = _spatialPak->ampLDataOffset * sizeof ( float );
		_spatialPak->ampLArray = createFloatDataArray ( _ampLFileName, _spatialPak->ampLArray, _spatialPak->ampLArraySize * sizeof ( float ),
									 					_spatialPak->ampLFilePos );

		snprintf ( viewName, sizeof ( viewName ), "Left reflection amplitude %s", utils->fileFromPath ( _ampLFileName ) );
		viewName [ sizeof ( viewName ) - 1 ] = 0;

		_spatialPak->ampLView = new GraphFltCmd ( viewName,
							TRUE, &CreateSpatialTemplateCmd::viewAmpLTemplateCmdCallback,
							mixwaveResources->numDigits(),
							_spatialPak->ampLArray,
							_spatialPak->ampLDataOffset,
							_currentSamples,
							_spatialPak->ampLDataSize,
							viewNumber,
							( void * ) this );

		_spatialPak->ampLGraphNum = viewNumber;
		_spatialPak->ampLView->setWindowSize ( width, height );
		_spatialPak->ampLView->autoClose ( False );	// wave window closes view
		viewNumber++;
	}
	
	delete ampLIn;

	strncpy ( _ampRFileName, _spatialPak->name, sizeof ( _ampRFileName ) );
	_ampRFileName [ sizeof ( _ampRFileName ) - 1 ] = 0;
	strncat ( _ampRFileName, "_ampr.flt", sizeof ( _ampRFileName ) - strlen ( _ampRFileName ) - 1 );	// s/b a resources-defined suffix
	_ampRFileName [ sizeof ( _ampRFileName ) - 1 ] = 0;

	FileAccess *ampRIn = new FileAccess ( _ampRFileName, "r+" );

	if ( ampRIn->errNum() != 0  )
		initError ( _ampRFileName, ampRIn->error() );
	else
	{
		_spatialPak->ampRDataSize = ( long ) ( float ) ampRIn->fileSize() / sizeof ( float );
		
		if ( (*wavePak)[_waveNum-1].pWave->totalSamples() == 0 )
			_amprScale = 1;
		else
			_amprScale = ( float ) _spatialPak->ampRDataSize / (*wavePak)[_waveNum-1].pWave->totalSamples();

		_spatialPak->ampRArraySize = ( long ) ( ( ( float ) waveSamples * _amprScale ) + 0.5 );

		if ( _spatialPak->ampRArraySize > _spatialPak->ampRDataSize )
			_spatialPak->ampRArraySize = _spatialPak->ampRDataSize;

		_spatialPak->ampRFilePos = _spatialPak->ampRDataOffset * sizeof ( float );
		_spatialPak->ampRArray = createFloatDataArray ( _ampRFileName, _spatialPak->ampRArray, _spatialPak->ampRArraySize * sizeof ( float ),
									 					_spatialPak->ampRFilePos );

		snprintf ( viewName, sizeof ( viewName ), "Right reflection amplitude %s", utils->fileFromPath ( _ampRFileName ) );
		viewName [ sizeof ( viewName ) - 1 ] = 0;

		_spatialPak->ampRView = new GraphFltCmd ( viewName,
							TRUE, &CreateSpatialTemplateCmd::viewAmpRTemplateCmdCallback,
							mixwaveResources->numDigits(),
							_spatialPak->ampRArray,
							_spatialPak->ampRDataOffset,
							_currentSamples,
							_spatialPak->ampRDataSize,
							viewNumber,
							( void * ) this );

		_spatialPak->ampRGraphNum = viewNumber;
		_spatialPak->ampRView->setWindowSize ( width, height );
		_spatialPak->ampRView->autoClose ( False );	// wave window closes view
		viewNumber++;
	}
	
	delete ampRIn;

	// text box objects
	_panTypeEdit = new LineEdit ( "Pan function", TRUE,
					&CreateSpatialTemplateCmd::panTypeEnteredCallback,
					&CreateSpatialTemplateCmd::panTypeCancelledCallback,
					( void * ) this );


	_filterTypeEdit = new LineEdit ( "Filter function", TRUE,
					&CreateSpatialTemplateCmd::filterTypeEnteredCallback,
					&CreateSpatialTemplateCmd::filterTypeCancelledCallback,
					( void * ) this );

	_angularSepEdit = new LineEdit ( "Angular separation", TRUE,
					&CreateSpatialTemplateCmd::angularSepEnteredCallback,
					&CreateSpatialTemplateCmd::angularSepCancelledCallback,
					( void * ) this );

	_signalScaleEdit = new LineEdit ( "Signal scaling", TRUE,
					&CreateSpatialTemplateCmd::signalScaleEnteredCallback,
					&CreateSpatialTemplateCmd::signalScaleCancelledCallback,
					( void * ) this );

	_detuneEdit = new LineEdit ( "Detune", TRUE,
					&CreateSpatialTemplateCmd::detuneEnteredCallback,
					&CreateSpatialTemplateCmd::detuneCancelledCallback,
					( void * ) this );

	_phaserEdit = new LineEdit ( "Phaser", TRUE,
					&CreateSpatialTemplateCmd::phaserEnteredCallback,
					&CreateSpatialTemplateCmd::phaserCancelledCallback,
					( void * ) this );

	_panLTypeEdit = new LineEdit ( "LR pan function", TRUE,
					&CreateSpatialTemplateCmd::panLTypeEnteredCallback,
					&CreateSpatialTemplateCmd::panLTypeCancelledCallback,
					( void * ) this );

	_panRTypeEdit = new LineEdit ( "RR pan function", TRUE,
					&CreateSpatialTemplateCmd::panRTypeEnteredCallback,
					&CreateSpatialTemplateCmd::panRTypeCancelledCallback,
					( void * ) this );

	_delayLEdit = new LineEdit ( "LR delay", TRUE,
					&CreateSpatialTemplateCmd::delayLEnteredCallback,
					&CreateSpatialTemplateCmd::delayLCancelledCallback,
					( void * ) this );

	_delayREdit = new LineEdit ( "RR delay", TRUE,
					&CreateSpatialTemplateCmd::delayREnteredCallback,
					&CreateSpatialTemplateCmd::delayRCancelledCallback,
					( void * ) this );

	_filterLTypeEdit = new LineEdit ( "LR filter function", TRUE,
					&CreateSpatialTemplateCmd::filterLTypeEnteredCallback,
					&CreateSpatialTemplateCmd::filterLTypeCancelledCallback,
					( void * ) this );

	_filterRTypeEdit = new LineEdit ( "RR filter function", TRUE,
					&CreateSpatialTemplateCmd::filterRTypeEnteredCallback,
					&CreateSpatialTemplateCmd::filterRTypeCancelledCallback,
					( void * ) this );

	_rvbRangeEdit = new LineEdit ( "Reverb range", TRUE,
					&CreateSpatialTemplateCmd::rvbRangeEnteredCallback,
					&CreateSpatialTemplateCmd::rvbRangeCancelledCallback,
					( void * ) this );

	_distRangeEdit = new LineEdit ( "Distance range", TRUE,
					&CreateSpatialTemplateCmd::distRangeEnteredCallback,
					&CreateSpatialTemplateCmd::distRangeCancelledCallback,
					( void * ) this );

	_bwRangeEdit = new LineEdit ( "Bandwidth range", TRUE,
					&CreateSpatialTemplateCmd::bwRangeEnteredCallback,
					&CreateSpatialTemplateCmd::bwRangeCancelledCallback,
					( void * ) this );

	_sweepRangeEdit = new LineEdit ( "Sweep freq. range", TRUE,
					&CreateSpatialTemplateCmd::sweepRangeEnteredCallback,
					&CreateSpatialTemplateCmd::sweepRangeCancelledCallback,
					( void * ) this );

	_dopplerRangeEdit = new LineEdit ( "Doppler shift range", TRUE,
					&CreateSpatialTemplateCmd::dopplerRangeEnteredCallback,
					&CreateSpatialTemplateCmd::dopplerRangeCancelledCallback,
					( void * ) this );

	_rvblRangeEdit = new LineEdit ( "L reverb range", TRUE,
					&CreateSpatialTemplateCmd::rvbLRangeEnteredCallback,
					&CreateSpatialTemplateCmd::rvbLRangeCancelledCallback,
					( void * ) this );

	_rvbrRangeEdit = new LineEdit ( "R reverb range", TRUE,
					&CreateSpatialTemplateCmd::rvbRRangeEnteredCallback,
					&CreateSpatialTemplateCmd::rvbRRangeCancelledCallback,
					( void * ) this );

	_bwlRangeEdit = new LineEdit ( "L bandwidth range", TRUE,
					&CreateSpatialTemplateCmd::bwLRangeEnteredCallback,
					&CreateSpatialTemplateCmd::bwLRangeCancelledCallback,
					( void * ) this );

	_bwrRangeEdit = new LineEdit ( "R bandwidth range", TRUE,
					&CreateSpatialTemplateCmd::bwRRangeEnteredCallback,
					&CreateSpatialTemplateCmd::bwRRangeCancelledCallback,
					( void * ) this );

	_sweeplRangeEdit = new LineEdit ( "L sweep freq. range", TRUE,
					&CreateSpatialTemplateCmd::sweepLRangeEnteredCallback,
					&CreateSpatialTemplateCmd::sweepLRangeCancelledCallback,
					( void * ) this );

	_sweeprRangeEdit = new LineEdit ( "R sweep freq. range", TRUE,
					&CreateSpatialTemplateCmd::sweepRRangeEnteredCallback,
					&CreateSpatialTemplateCmd::sweepRRangeCancelledCallback,
					( void * ) this );

	_amplRangeEdit = new LineEdit ( "L amplitude range", TRUE,
					&CreateSpatialTemplateCmd::ampLRangeEnteredCallback,
					&CreateSpatialTemplateCmd::ampLRangeCancelledCallback,
					( void * ) this );

	_amprRangeEdit = new LineEdit ( "R amplitude range", TRUE,
					&CreateSpatialTemplateCmd::ampRRangeEnteredCallback,
					&CreateSpatialTemplateCmd::ampRRangeCancelledCallback,
					( void * ) this );

	_rvbTailEdit = new LineEdit ( "Reverb tail time", TRUE,
					&CreateSpatialTemplateCmd::rvbTailEnteredCallback,
					&CreateSpatialTemplateCmd::rvbTailCancelledCallback,
					( void * ) this );

	_cssCmd = new CreateSpatialScoreCmd ( "createSpatialScore", TRUE, _waveNum, _spatialPak );

	float duration = ( float ) (*wavePak)[_waveNum-1].pWave->totalSamples() / ( float ) (*wavePak)[_waveNum-1].pWave->sampleRate();

	_runCsndCmd = new RunCsoundCmd ( "Csound", TRUE, _spatialPak->name, duration );

	_shapeCmd = new ShapeSpatialPakTemplateCmd ( "ShapeSpatialTemplate", TRUE, mixwaveResources->numDigits(),
													mixwaveResources->fpDigits() );
	_shapeCmdRealised = True;
	_shapeCmd->execute();

	delete utils;
	_viewRealised = True;
}


void CreateSpatialTemplateCmd::menuSelectionCallback ( void *clientData, unsigned int selectedItemNum, char *label )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;	// dummy obj

	if ( obj->_error )
		return;
		
	// Since the menu is static with reference to the name strings
	// (quitOnSelect is false), the reference may be invalid; check by name.

	if ( ( strcmp ( label, "Quit" ) == 0 ) || ( selectedItemNum == 0 ) )
	{
		obj->deleteMenuObjects();
		delete obj->_menu;
		obj->_menuRealised = False;
	}
	else
		obj->menuSelection ( label, selectedItemNum );
}


void CreateSpatialTemplateCmd::pathMenuSelectionCallback ( void *clientData, unsigned int selectedItemNum, char *label )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;	// dummy obj

	if ( obj->_error )
		return;

	if ( ( strcmp ( label, "Quit" ) == 0 ) || ( selectedItemNum == 0 ) )
	{
		obj->deletePathMenuObjects();	
		delete obj->_pathMenu;
		obj->_pathMenuRealised = False;
	}
	else
		obj->menuSelection ( label, selectedItemNum + 14 );	// menu offset
}


void CreateSpatialTemplateCmd::paramRangeMenuSelectionCallback ( void *clientData, unsigned int selectedItemNum, char *label )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;	// dummy obj

	if ( obj->_error )
		return;

	if ( ( strcmp ( label, "Quit" ) == 0 ) || ( selectedItemNum == 0 ) )
	{
		obj->deleteParamsRangeMenuObjects();
		delete obj->_paramRangeMenu;
		obj->_paramRangeMenuRealised = False;
	}
	else
		obj->menuSelection ( label, selectedItemNum + 30 );	// menu offset
}


void CreateSpatialTemplateCmd::menuSelection ( char *label, unsigned selectedItem )
{
	switch ( selectedItem )
	{
		case 1:	// write score file (... and process wave file? )
			_cssCmd->execute ( &CreateSpatialTemplateCmd::spatialScoreCreatedCallback, ( void * ) this );
			break;

		case 2:	// reverb envelope view
			if ( _rvbViewRealised == False )
			{
				_spatialPak->rvbView->execute();
				_spatialPak->rvbView->setSelection ( _spatialPak->rvbStartSel, _spatialPak->rvbEndSel );
				_spatialPak->rvbView->setDatumValueBounds ( _spatialPak->rvbMin, _spatialPak->rvbMax );

				if ( _spatialPak->rvbMin < 0 )
					_spatialPak->rvbView->centreZero ( True );	// set to signed data type
				else
					_spatialPak->rvbView->centreZero ( False );

				_rvbViewRealised = True;
			}
			else	// note we can't do this until the view has the data
				_shapeCmd->templateID ( _rvbFileName, _spatialPak->rvbStartRange, _spatialPak->rvbEndRange,
							_spatialPak->rvbStartSel, _spatialPak->rvbEndSel,
							_spatialPak->rvbXCurrent, _spatialPak->rvbMin,
							_spatialPak->rvbMax, NULL, _spatialPak->rvbView,
							NULL, _spatialPak->rvbArray, 1,
							_spatialPak->rvbGraphNum );

			break;
		
		case 3:	// sound source distance-from-subject envelope view
			if ( _rangeViewRealised == False )
			{
 				_spatialPak->rangeView->execute();
				_spatialPak->rangeView->setSelection ( _spatialPak->rangeStartSel, _spatialPak->rangeEndSel );
				_spatialPak->rangeView->setDatumValueBounds ( _spatialPak->rangeMin, _spatialPak->rangeMax );

				if ( _spatialPak->rangeMin < 0 )
					_spatialPak->rangeView->centreZero ( True );	// set to signed data type
				else
					_spatialPak->rangeView->centreZero ( False );

				_rangeViewRealised = True;
			}
			else			
				_shapeCmd->templateID ( _rangeFileName, _spatialPak->rangeStartRange, _spatialPak->rangeEndRange,
							_spatialPak->rangeStartSel, _spatialPak->rangeEndSel,
							_spatialPak->rangeXCurrent, _spatialPak->rangeMin,
							_spatialPak->rangeMax, NULL, _spatialPak->rangeView,
							NULL, _spatialPak->rangeArray, 1,
							_spatialPak->rangeGraphNum );

			break;

		case 4:	// bandwidth envelope view
			if ( _bwViewRealised == False )
			{
				_spatialPak->bwView->execute();
				_spatialPak->bwView->setSelection ( _spatialPak->bwStartSel, _spatialPak->bwEndSel );
				_spatialPak->bwView->setDatumValueBounds ( _spatialPak->bwMin, _spatialPak->bwMax );

				if ( _spatialPak->bwMin < 0 )
					_spatialPak->bwView->centreZero ( True );	// set to signed data type
				else
					_spatialPak->bwView->centreZero ( False );

				_bwViewRealised = True;
			}
			else			
				_shapeCmd->templateID ( _bwFileName, _spatialPak->bwStartRange, _spatialPak->bwEndRange,
							_spatialPak->bwStartSel, _spatialPak->bwEndSel,
							_spatialPak->bwXCurrent, _spatialPak->bwMin,
							_spatialPak->bwMax, NULL, _spatialPak->bwView,
							NULL, _spatialPak->bwArray, 1,
							_spatialPak->bwGraphNum );

			break;

		case 5:	// sweep freq. envelope view
			if ( _sweepViewRealised == False )
			{
				_spatialPak->sweepView->execute();
				_spatialPak->sweepView->setSelection ( _spatialPak->sweepStartSel, _spatialPak->sweepEndSel );
				_spatialPak->sweepView->setDatumValueBounds ( _spatialPak->sweepMin, _spatialPak->sweepMax );

				if ( _spatialPak->sweepMin < 0 )
					_spatialPak->sweepView->centreZero ( True );	// set to signed data type
				else
					_spatialPak->sweepView->centreZero ( False );

				_sweepViewRealised = True;
			}
			else			
				_shapeCmd->templateID ( _sweepFileName, _spatialPak->sweepStartRange, _spatialPak->sweepEndRange,
							_spatialPak->sweepStartSel, _spatialPak->sweepEndSel,
							_spatialPak->sweepXCurrent, _spatialPak->sweepMin,
							_spatialPak->sweepMax, NULL, _spatialPak->sweepView,
							NULL, _spatialPak->sweepArray, 1,
							_spatialPak->sweepGraphNum );

			break;

		case 6:	// pan envelope view
			if ( _panViewRealised == False )
			{
				_spatialPak->panView->execute();
				_spatialPak->panView->setSelection ( _spatialPak->panStartSel, _spatialPak->panEndSel );
				_spatialPak->panView->setDatumValueBounds ( _spatialPak->panMin, _spatialPak->panMax );

				if ( _spatialPak->panMin < 0 )
					_spatialPak->panView->centreZero ( True );	// set to signed data type
				else
					_spatialPak->panView->centreZero ( False );

				_panViewRealised = True;
			}
			else			
				_shapeCmd->templateID ( _panFileName, _spatialPak->panStartRange, _spatialPak->panEndRange,
							_spatialPak->panStartSel, _spatialPak->panEndSel,
							_spatialPak->panXCurrent, _spatialPak->panMin,
							_spatialPak->panMax, NULL, _spatialPak->panView,
							NULL, _spatialPak->panArray, 1,
							_spatialPak->panGraphNum );

			break;

		case 7:	// doppler shift envelope view
			if ( _dopplerViewRealised == False )
			{
				_spatialPak->dopplerView->execute();
				_spatialPak->dopplerView->setSelection ( _spatialPak->dopplerStartSel, _spatialPak->dopplerEndSel );
				_spatialPak->dopplerView->setDatumValueBounds ( _spatialPak->dopplerMin, _spatialPak->dopplerMax );

				if ( _spatialPak->dopplerMin < 0 )
					_spatialPak->dopplerView->centreZero ( True );	// set to signed data type
				else
					_spatialPak->dopplerView->centreZero ( False );

				_dopplerViewRealised = True;
			}
			else			
				_shapeCmd->templateID ( _dplrFileName, _spatialPak->dopplerStartRange, _spatialPak->dopplerEndRange,
							_spatialPak->dopplerStartSel, _spatialPak->dopplerEndSel,
							_spatialPak->dopplerXCurrent, _spatialPak->dopplerMin,
							_spatialPak->dopplerMax, NULL, _spatialPak->dopplerView,
							NULL, _spatialPak->dopplerArray, 1,
							_spatialPak->dopplerGraphNum );
			break;

		case 8:	// pan function type text box
			if ( _panTypeEditRealised == False )
			{
				char buf[20];
				char help[1024];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%u", _spatialPak->panType );
				buf [sizeof ( buf ) - 1] = 0;

				snprintf ( help, sizeof ( help ), "Specifies pan function type; 0 <= pan fn \
<= 2: current pan type %u (", _spatialPak->panType );
				help [ sizeof ( help ) - 1 ] = 0;

				if ( _spatialPak->panType == 0 )
					strncat ( help, "constant-power).", sizeof ( help ) - strlen ( help ) - 1 );

				if ( _spatialPak->panType == 1 )
					strncat ( help, "Gogins, M.).", sizeof ( help ) - strlen ( help ) - 1 );

				if ( _spatialPak->panType == 2 )
					strncat ( help, "Cascone, K.).", sizeof ( help ) - strlen ( help ) - 1 );

				help [ sizeof ( help ) - 1 ] = 0;

				_panTypeEdit->setHelp ( help );
				_panTypeEdit->setColumns ( strlen ( buf ) );
				_panTypeEdit->execute();
				_panTypeEdit->setText ( buf );
				_panTypeEditRealised = True;
			}
			
			break;

		case 9:	// filter function type text box
			if ( _filterTypeEditRealised == False )
			{
				char buf[20];
				char help[1024];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%u", _spatialPak->filterType );
				buf [ sizeof ( buf ) - 1 ] = 0;
				
				snprintf ( help, sizeof ( help ), "Specifies filter type; 0 <= filter fn \
<= 1: current filter type %u (", _spatialPak->filterType );

				if ( _spatialPak->filterType == 0 )
					strncat ( help, "bandpass).", sizeof ( help ) - strlen ( help ) - 1 );

				if ( _spatialPak->filterType == 1 )
					strncat ( help, "notch).", sizeof ( help ) - strlen ( help ) - 1 );

				help [ sizeof ( help ) - 1 ] = 0;

				_filterTypeEdit->setHelp ( help );
				_filterTypeEdit->setColumns ( strlen ( buf ) );
				_filterTypeEdit->execute();
				_filterTypeEdit->setText ( buf );
				_filterTypeEditRealised = True;
			}
			
			break;

		case 10:	// sound reproducer angular separation text box
			if ( _angularSepEditRealised == False )
			{
				char buf[20];
				char help[1024];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%f", _spatialPak->angularSep );
				buf [sizeof ( buf ) - 1] = 0;

				snprintf ( help, sizeof ( help ), "Specifies angular separation \
of sound emitters; 0 <= angular position <= 1: 0 = mono, 0.5 = stereo \
(speakers), 1 = stereo (headphones)." );
				help [ sizeof ( help ) - 1 ] = 0;

				_angularSepEdit->setHelp ( help );
				_angularSepEdit->setColumns ( strlen ( buf ) );
				_angularSepEdit->execute();
				_angularSepEdit->setText ( buf );
				_angularSepEditRealised = True;
			}
			
			break;

		case 11:	// signal scaling text box
			if ( _signalScaleEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%f", _spatialPak->signalScale );
				buf [sizeof ( buf ) - 1] = 0;

				_signalScaleEdit->setColumns ( strlen ( buf ) );
				_signalScaleEdit->execute();
				_signalScaleEdit->setText ( buf );
				_signalScaleEditRealised = True;
			}
			
			break;

		case 12:	// detune text box
			if ( _detuneEditRealised == False )
			{
// if phaser off, no detune
				char buf[20];
				char help[1024];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%f", _spatialPak->detune );
				buf [sizeof ( buf ) - 1] = 0;

				snprintf ( help, sizeof ( help ), "Specifies detune value used \
in phasing effect applied to doppler shift; detune >= 0. Requires doppler < \
1, detune > 0 and phaser on: detune is " );

				if ( _spatialPak->detune == 0 )
					strncat ( help, "OFF;", sizeof ( help ) - strlen ( help ) - 1 );
				else
					strncat ( help, "ON;", sizeof ( help ) - strlen ( help ) - 1 );

				help [ sizeof ( help ) - 1 ] = 0;
				
				strncat ( help, " phaser is ", sizeof ( help ) - strlen ( help ) - 1 );
				help [ sizeof ( help ) - 1 ] = 0;

				if ( _spatialPak->phaser == 0 )
					strncat ( help, "OFF.", sizeof ( help ) - strlen ( help ) - 1 );
				else
					strncat ( help, "ON.", sizeof ( help ) - strlen ( help ) - 1 );

				help [ sizeof ( help ) - 1 ] = 0;

				_detuneEdit->setHelp ( help );
				_detuneEdit->setColumns ( strlen ( buf ) );
				_detuneEdit->execute();
				_detuneEdit->setText ( buf );
				_detuneEditRealised = True;
			}
			
			break;

		case 13:	// phaser text box
			if ( _phaserEditRealised == False )
			{
				char buf[20];
				char help[1024];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%u", _spatialPak->phaser );
				buf [sizeof ( buf ) - 1] = 0;

				snprintf ( help, sizeof ( help ), "Specifies phaser use \
activating detune effect on doppler shift; values 0 (off), 1 (on). Requires doppler < \
1, detune > 0 and phaser on: detune is " );

				if ( _spatialPak->phaser == 0 )
					strncat ( help, "OFF;", sizeof ( help ) - strlen ( help ) - 1 );
				else
					strncat ( help, "ON;", sizeof ( help ) - strlen ( help ) - 1 );

				help [ sizeof ( help ) - 1 ] = 0;
				
				strncat ( help, " phaser is ", sizeof ( help ) - strlen ( help ) - 1 );
				help [ sizeof ( help ) - 1 ] = 0;

				if ( _spatialPak->phaser == 0 )
					strncat ( help, "OFF.", sizeof ( help ) - strlen ( help ) - 1 );
				else
					strncat ( help, "ON.", sizeof ( help ) - strlen ( help ) - 1 );

				help [ sizeof ( help ) - 1 ] = 0;

				_phaserEdit->setHelp ( help );
				_phaserEdit->setColumns ( strlen ( buf ) );
				_phaserEdit->execute();
				_phaserEdit->setText ( buf );
				_phaserEditRealised = True;
			}
			
			break;

		case 14:	// reverb tail text box
			if ( _rvbTailEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%f", _spatialPak->rvbTailTime );
				buf [sizeof ( buf ) - 1] = 0;

				_rvbTailEdit->setColumns ( strlen ( buf ) );
				_rvbTailEdit->execute();
				_rvbTailEdit->setText ( buf );
				_rvbTailEditRealised = True;
			}
			
			break;

		case 15:	// left reflection reverb envelope view
			if ( _rvbLViewRealised == False )
			{
				_spatialPak->rvbLView->execute();
				_spatialPak->rvbLView->setSelection ( _spatialPak->rvbLStartSel, _spatialPak->rvbLEndSel );
				_spatialPak->rvbLView->setDatumValueBounds ( _spatialPak->rvbLMin, _spatialPak->rvbLMax );

				if ( _spatialPak->rvbLMin < 0 )
					_spatialPak->rvbLView->centreZero ( True );	// set to signed data type
				else
					_spatialPak->rvbLView->centreZero ( False );

				_rvbLViewRealised = True;
			}
			else			
				_shapeCmd->templateID ( _rvbLFileName, _spatialPak->rvbLStartRange, _spatialPak->rvbLEndRange,
							_spatialPak->rvbLStartSel, _spatialPak->rvbLEndSel,
							_spatialPak->rvbLXCurrent, _spatialPak->rvbLMin,
							_spatialPak->rvbLMax, NULL, _spatialPak->rvbLView,
							NULL, _spatialPak->rvbLArray, 1,
							_spatialPak->rvbLGraphNum );

			break;

		case 16:	// right reflection reverb envelope view
			if ( _rvbRViewRealised == False )
			{
				_spatialPak->rvbRView->execute();
				_spatialPak->rvbRView->setSelection ( _spatialPak->rvbRStartSel, _spatialPak->rvbREndSel );
				_spatialPak->rvbRView->setDatumValueBounds ( _spatialPak->rvbRMin, _spatialPak->rvbRMax );

				if ( _spatialPak->rvbRMin < 0 )
					_spatialPak->rvbRView->centreZero ( True );	// set to signed data type
				else
					_spatialPak->rvbRView->centreZero ( False );

				_rvbRViewRealised = True;
			}
			else
				_shapeCmd->templateID ( _rvbRFileName, _spatialPak->rvbRStartRange, _spatialPak->rvbREndRange,
							_spatialPak->rvbRStartSel, _spatialPak->rvbREndSel,
							_spatialPak->rvbRXCurrent, _spatialPak->rvbRMin,
							_spatialPak->rvbRMax, NULL, _spatialPak->rvbRView,
							NULL, _spatialPak->rvbRArray, 1,
							_spatialPak->rvbRGraphNum );

			break;
			
		case 17:	// left reflection pan envelope view
			if ( _panLViewRealised == False )
			{
				_spatialPak->panLView->execute();
				_spatialPak->panLView->setSelection ( _spatialPak->panLStartSel, _spatialPak->panLEndSel );
				_spatialPak->panLView->setDatumValueBounds ( _spatialPak->panLMin, _spatialPak->panLMax );

				if ( _spatialPak->panLMin < 0 )
					_spatialPak->panLView->centreZero ( True );	// set to signed data type
				else
					_spatialPak->panLView->centreZero ( False );

				_panLViewRealised = True;
			}
			else
				_shapeCmd->templateID ( _panLFileName, _spatialPak->panLStartRange, _spatialPak->panLEndRange,
							_spatialPak->panLStartSel, _spatialPak->panLEndSel,
							_spatialPak->panLXCurrent, _spatialPak->panLMin,
							_spatialPak->panLMax, NULL, _spatialPak->panLView,
							NULL, _spatialPak->panLArray, 1,
							_spatialPak->panLGraphNum );

			break;
			
		case 18:	// right reflection pan envelope view
			if ( _panRViewRealised == False )
			{
				_spatialPak->panRView->execute();
				_spatialPak->panRView->setSelection ( _spatialPak->panRStartSel, _spatialPak->panREndSel );
				_spatialPak->panRView->setDatumValueBounds ( _spatialPak->panRMin, _spatialPak->panRMax );

				if ( _spatialPak->panRMin < 0 )
					_spatialPak->panRView->centreZero ( True );	// set to signed data type
				else
					_spatialPak->panRView->centreZero ( False );

				_panRViewRealised = True;
			}
			else
				_shapeCmd->templateID ( _panRFileName, _spatialPak->panRStartRange, _spatialPak->panREndRange,
							_spatialPak->panRStartSel, _spatialPak->panREndSel,
							_spatialPak->panRXCurrent, _spatialPak->panRMin,
							_spatialPak->panRMax, NULL, _spatialPak->panRView,
							NULL, _spatialPak->panRArray, 1,
							_spatialPak->panRGraphNum );

			break;

		case 19:	// panLTypeEdit text box
			if ( _panLTypeEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%u", _spatialPak->panLType );
				buf [sizeof ( buf ) - 1] = 0;

				_panLTypeEdit->setColumns ( strlen ( buf ) );
				_panLTypeEdit->execute();
				_panLTypeEdit->setText ( buf );
				_panLTypeEditRealised = True;
			}
			
			break;

		case 20:	// panRTypeEdit text box
			if ( _panRTypeEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%u", _spatialPak->panRType );
				buf [sizeof ( buf ) - 1] = 0;

				_panRTypeEdit->setColumns ( strlen ( buf ) );
				_panRTypeEdit->execute();
				_panRTypeEdit->setText ( buf );
				_panRTypeEditRealised = True;
			}
			
			break;

		case 21:	// left reflection bandwidth envelope view
			if ( _bwLViewRealised == False )
			{
				_spatialPak->bwLView->execute();
				_spatialPak->bwLView->setSelection ( _spatialPak->bwLStartSel, _spatialPak->bwLEndSel );
				_spatialPak->bwLView->setDatumValueBounds ( _spatialPak->bwLMin, _spatialPak->bwLMax );

				if ( _spatialPak->bwLMin < 0 )
					_spatialPak->bwLView->centreZero ( True );	// set to signed data type
				else
					_spatialPak->bwLView->centreZero ( False );

				_bwLViewRealised = True;
			}
			else
				_shapeCmd->templateID ( _bwLFileName, _spatialPak->bwLStartRange, _spatialPak->bwLEndRange,
							_spatialPak->bwLStartSel, _spatialPak->bwLEndSel,
							_spatialPak->bwLXCurrent, _spatialPak->bwLMin,
							_spatialPak->bwLMax, NULL, _spatialPak->bwLView,
							NULL, _spatialPak->bwLArray, 1,
							_spatialPak->bwLGraphNum );

			break;

		case 22:	// right reflection bandwidth envelope view
			if ( _bwRViewRealised == False )
			{
				_spatialPak->bwRView->execute();
				_spatialPak->bwRView->setSelection ( _spatialPak->bwRStartSel, _spatialPak->bwREndSel );
				_spatialPak->bwRView->setDatumValueBounds ( _spatialPak->bwRMin, _spatialPak->bwRMax );

				if ( _spatialPak->bwRMin < 0 )
					_spatialPak->bwRView->centreZero ( True );	// set to signed data type
				else
					_spatialPak->bwRView->centreZero ( False );

				_bwRViewRealised = True;
			}
			else
				_shapeCmd->templateID ( _bwRFileName, _spatialPak->bwRStartRange, _spatialPak->bwREndRange,
							_spatialPak->bwRStartSel, _spatialPak->bwREndSel,
							_spatialPak->bwRXCurrent, _spatialPak->bwRMin,
							_spatialPak->bwRMax, NULL, _spatialPak->bwRView,
							NULL, _spatialPak->bwRArray, 1,
							_spatialPak->bwRGraphNum );

			break;

		case 23:	// left reflection sweep freq. envelope view
			if ( _sweepLViewRealised == False )
			{
				_spatialPak->sweepLView->execute();
				_spatialPak->sweepLView->setSelection ( _spatialPak->sweepLStartSel, _spatialPak->sweepLEndSel );
				_spatialPak->sweepLView->setDatumValueBounds ( _spatialPak->sweepLMin, _spatialPak->sweepLMax );

				if ( _spatialPak->sweepLMin < 0 )
					_spatialPak->sweepLView->centreZero ( True );	// set to signed data type
				else
					_spatialPak->sweepLView->centreZero ( False );

				_sweepLViewRealised = True;
			}
			else
				_shapeCmd->templateID ( _sweepLFileName, _spatialPak->sweepLStartRange, _spatialPak->sweepLEndRange,
							_spatialPak->sweepLStartSel, _spatialPak->sweepLEndSel,
							_spatialPak->sweepLXCurrent, _spatialPak->sweepLMin,
							_spatialPak->sweepLMax, NULL, _spatialPak->sweepLView,
							NULL, _spatialPak->sweepLArray, 1,
							_spatialPak->sweepLGraphNum );

			break;

		case 24:	// right reflection sweep freq. envelope view
			if ( _sweepRViewRealised == False )
			{
				_spatialPak->sweepRView->execute();
				_spatialPak->sweepRView->setSelection ( _spatialPak->sweepRStartSel, _spatialPak->sweepREndSel );
				_spatialPak->sweepRView->setDatumValueBounds ( _spatialPak->sweepRMin, _spatialPak->sweepRMax );

				if ( _spatialPak->sweepRMin < 0 )
					_spatialPak->sweepRView->centreZero ( True );	// set to signed data type
				else
					_spatialPak->sweepRView->centreZero ( False );

				_sweepRViewRealised = True;
			}
			else
				_shapeCmd->templateID ( _sweepRFileName, _spatialPak->sweepRStartRange, _spatialPak->sweepREndRange,
							_spatialPak->sweepRStartSel, _spatialPak->sweepREndSel,
							_spatialPak->sweepRXCurrent, _spatialPak->sweepRMin,
							_spatialPak->sweepRMax, NULL, _spatialPak->sweepRView,
							NULL, _spatialPak->sweepRArray, 1,
							_spatialPak->sweepRGraphNum );

			break;

		case 25:	// left reflection amplitude envelope view
			if ( _ampLViewRealised == False )
			{
				_spatialPak->ampLView->execute();
				_spatialPak->ampLView->setSelection ( _spatialPak->ampLStartSel, _spatialPak->ampLEndSel );
				_spatialPak->ampLView->setDatumValueBounds ( _spatialPak->ampLMin, _spatialPak->ampLMax );

				if ( _spatialPak->ampLMin < 0 )
					_spatialPak->ampLView->centreZero ( True );	// set to signed data type
				else
					_spatialPak->ampLView->centreZero ( False );

				_ampLViewRealised = True;
			}
			else
				_shapeCmd->templateID ( _ampLFileName, _spatialPak->ampLStartRange, _spatialPak->ampLEndRange,
							_spatialPak->ampLStartSel, _spatialPak->ampLEndSel,
							_spatialPak->ampLXCurrent, _spatialPak->ampLMin,
							_spatialPak->ampLMax, NULL, _spatialPak->ampLView,
							NULL, _spatialPak->ampLArray, 1,
							_spatialPak->ampLGraphNum );

			break;

		case 26:	// right reflection amplitude envelope view
			if ( _ampRViewRealised == False )
			{
				_spatialPak->ampRView->execute();
				_spatialPak->ampRView->setSelection ( _spatialPak->ampRStartSel, _spatialPak->ampREndSel );
				_spatialPak->ampRView->setDatumValueBounds ( _spatialPak->ampRMin, _spatialPak->ampRMax );

				if ( _spatialPak->ampRMin < 0 )
					_spatialPak->ampRView->centreZero ( True );	// set to signed data type
				else
					_spatialPak->ampRView->centreZero ( False );

				_ampRViewRealised = True;
			}
			else
				_shapeCmd->templateID ( _ampRFileName, _spatialPak->ampRStartRange, _spatialPak->ampREndRange,
							_spatialPak->ampRStartSel, _spatialPak->ampREndSel,
							_spatialPak->ampRXCurrent, _spatialPak->ampRMin,
							_spatialPak->ampRMax, NULL, _spatialPak->ampRView,
							NULL, _spatialPak->ampRArray, 1,
							_spatialPak->ampRGraphNum );

			break;

		case 27:	// delayLEdit text box
			if ( _delayLEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%u", _spatialPak->panLType );
				buf [sizeof ( buf ) - 1] = 0;

				_delayLEdit->setColumns ( strlen ( buf ) );
				_delayLEdit->execute();
				_delayLEdit->setText ( buf );
				_delayLEditRealised = True;
			}
			
			break;

		case 28:	// delayREdit text box
			if ( _delayREditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%u", _spatialPak->panRType );
				buf [sizeof ( buf ) - 1] = 0;

				_delayREdit->setColumns ( strlen ( buf ) );
				_delayREdit->execute();
				_delayREdit->setText ( buf );
				_delayREditRealised = True;
			}
			
			break;

		case 29:	// filterLTypeEdit text box
			if ( _filterLTypeEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				if ( _spatialPak->filterLType == 0 )
					strcpy ( buf, "band" );
				else
					if ( _spatialPak->filterLType == 1 ) 
						strcpy ( buf, "notch" );	

				_filterLTypeEdit->setColumns ( strlen ( buf ) );
				_filterLTypeEdit->execute();
				_filterLTypeEdit->setText ( buf );
				_filterLTypeEditRealised = True;
			}
			
			break;

		case 30:	// filterRTypeEdit text box
			if ( _filterRTypeEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				if ( _spatialPak->filterRType == 0 )
					strcpy ( buf, "band" );
				else
					if ( _spatialPak->filterRType == 1 ) 
						strcpy ( buf, "notch" );	

				_filterRTypeEdit->setColumns ( strlen ( buf ) );
				_filterRTypeEdit->execute();
				_filterRTypeEdit->setText ( buf );
				_filterRTypeEditRealised = True;
			}
			
			break;

		case 31:	// rvbRangeEdit text box
			if ( _rvbRangeEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%f,%f", _spatialPak->rvbMin, _spatialPak->rvbMax );
				buf [sizeof ( buf ) - 1] = 0;

				_rvbRangeEdit->setColumns ( strlen ( buf ) );
				_rvbRangeEdit->execute();
				_rvbRangeEdit->setText ( buf );
				_rvbRangeEditRealised = True;
			}
			
			break;

		case 32:	// distRangeEdit text box
			if ( _distRangeEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%f,%f", _spatialPak->rangeMin, _spatialPak->rangeMax );
				buf [sizeof ( buf ) - 1] = 0;

				_distRangeEdit->setColumns ( strlen ( buf ) );
				_distRangeEdit->execute();
				_distRangeEdit->setText ( buf );
				_distRangeEditRealised = True;
			}
			
			break;

		case 33:	// bwRangeEdit text box
			if ( _bwRangeEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%f,%f", _spatialPak->bwMin, _spatialPak->bwMax );
				buf [sizeof ( buf ) - 1] = 0;

				_bwRangeEdit->setColumns ( strlen ( buf ) );
				_bwRangeEdit->execute();
				_bwRangeEdit->setText ( buf );
				_bwRangeEditRealised = True;
			}
			
			break;

		case 34:	// sweepRangeEdit text box
			if ( _sweepRangeEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%f,%f", _spatialPak->sweepMin, _spatialPak->sweepMax );
				buf [sizeof ( buf ) - 1] = 0;

				_sweepRangeEdit->setColumns ( strlen ( buf ) );
				_sweepRangeEdit->execute();
				_sweepRangeEdit->setText ( buf );
				_sweepRangeEditRealised = True;
			}
			
			break;

		case 35:	// dopplerRangeEdit text box
			if ( _dopplerRangeEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%f,%f", _spatialPak->dopplerMin, _spatialPak->dopplerMax );
				buf [sizeof ( buf ) - 1] = 0;

				_dopplerRangeEdit->setColumns ( strlen ( buf ) );
				_dopplerRangeEdit->execute();
				_dopplerRangeEdit->setText ( buf );
				_dopplerRangeEditRealised = True;
			}
			
			break;

		case 36:	// rvblRangeEdit text box
			if ( _rvblRangeEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%f,%f", _spatialPak->rvbLMin, _spatialPak->rvbLMax );
				buf [sizeof ( buf ) - 1] = 0;

				_rvblRangeEdit->setColumns ( strlen ( buf ) );
				_rvblRangeEdit->execute();
				_rvblRangeEdit->setText ( buf );
				_rvblRangeEditRealised = True;
			}
			
			break;

		case 37:	// rvbrRangeEdit text box
			if ( _rvbrRangeEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%f,%f", _spatialPak->rvbRMin, _spatialPak->rvbRMax );
				buf [sizeof ( buf ) - 1] = 0;

				_rvbrRangeEdit->setColumns ( strlen ( buf ) );
				_rvbrRangeEdit->execute();
				_rvbrRangeEdit->setText ( buf );
				_rvbrRangeEditRealised = True;
			}
			
			break;

		case 38:	// bwlRangeEdit text box
			if ( _bwlRangeEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%f,%f", _spatialPak->bwLMin, _spatialPak->bwLMax );
				buf [sizeof ( buf ) - 1] = 0;

				_bwlRangeEdit->setColumns ( strlen ( buf ) );
				_bwlRangeEdit->execute();
				_bwlRangeEdit->setText ( buf );
				_bwlRangeEditRealised = True;
			}
			
			break;

		case 39:	// bwrRangeEdit text box
			if ( _bwrRangeEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%f,%f", _spatialPak->bwRMin, _spatialPak->bwRMax );
				buf [sizeof ( buf ) - 1] = 0;

				_bwrRangeEdit->setColumns ( strlen ( buf ) );
				_bwrRangeEdit->execute();
				_bwrRangeEdit->setText ( buf );
				_bwrRangeEditRealised = True;
			}
			
			break;

		case 40:	// sweeplRangeEdit text box
			if ( _sweeplRangeEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%f,%f", _spatialPak->sweepLMin, _spatialPak->sweepLMax );
				buf [sizeof ( buf ) - 1] = 0;

				_sweeplRangeEdit->setColumns ( strlen ( buf ) );
				_sweeplRangeEdit->execute();
				_sweeplRangeEdit->setText ( buf );
				_sweeplRangeEditRealised = True;
			}
			
			break;

		case 41:	// sweeprRangeEdit text box
			if ( _sweeprRangeEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%f,%f", _spatialPak->sweepRMin, _spatialPak->sweepRMax );
				buf [sizeof ( buf ) - 1] = 0;

				_sweeprRangeEdit->setColumns ( strlen ( buf ) );
				_sweeprRangeEdit->execute();
				_sweeprRangeEdit->setText ( buf );
				_sweeprRangeEditRealised = True;
			}
			
			break;

		case 42:	// amplRangeEdit text box
			if ( _amplRangeEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%f,%f", _spatialPak->ampLMin, _spatialPak->ampLMax );
				buf [sizeof ( buf ) - 1] = 0;

				_amplRangeEdit->setColumns ( strlen ( buf ) );
				_amplRangeEdit->execute();
				_amplRangeEdit->setText ( buf );
				_amplRangeEditRealised = True;
			}
			
			break;

		case 43:	// amprRangeEdit text box
			if ( _amprRangeEditRealised == False )
			{
				char buf[20];
 				// retrieve resource to be displayed
				snprintf ( buf, sizeof ( buf ), "%f,%f", _spatialPak->ampRMin, _spatialPak->ampRMax );
				buf [sizeof ( buf ) - 1] = 0;

				_amprRangeEdit->setColumns ( strlen ( buf ) );
				_amprRangeEdit->execute();
				_amprRangeEdit->setText ( buf );
				_amprRangeEditRealised = True;
			}
			
			break;
	}
}


void CreateSpatialTemplateCmd::waveViewCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, long yCurrent, Boolean reload,
								unsigned waveNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	GraphIntCmd *gc = ( GraphIntCmd * ) cmd;

	obj->waveViewCallbackFinished ( gc, start, end, selStart, selEnd,
										xCurrent, yCurrent, reload, waveNum, callbackType );

	if ( callbackType != 0 )
	{
		obj->exit();

		if ( obj->_callback )
		{
			if ( obj->_error )
				obj->_callback ( obj, obj->_clientData, 3 );	// kill all files
			else
				obj->_callback ( obj, obj->_clientData, callbackType );
		}
	}
}


void CreateSpatialTemplateCmd::waveDataCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, long yCurrent, Boolean reload,
								unsigned waveNum, unsigned callbackType )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
//	WaveDataCmd *wd = ( WaveDataCmd * ) cmd;	// unused
		
	obj->waveViewCallbackFinished ( (*wavePak)[waveNum-1].graphCmd, start, end, selStart, selEnd,
										xCurrent, yCurrent, reload, waveNum, callbackType );
	
	// 'output' from data obj dependent on GraphCmd obj; used to drive GC 'input'
	// if data has been changed here, reflect this in the GraphCmd obj
	if ( reload == True )
		(*wavePak)[waveNum-1].dataOffset = start;	// start posn of new data

	if ( (*wavePak)[waveNum-1].graphCmd->redisplay ( start, end, selStart, selEnd, xCurrent,
														(*wavePak)[waveNum-1].dataOffset ) != 0 )
	{
		char msg[BUFSIZE];
		snprintf ( msg, sizeof ( msg ), "%s\nCreateSpatialTemplateCmd view update failed. No data to display.",
					(*wavePak)[waveNum-1].pWave->name() );
		msg [ sizeof ( msg ) - 1 ] = 0;
	}
}


void CreateSpatialTemplateCmd::waveViewCallbackFinished ( GraphIntCmd *gc, long start, long end,
								long selStart, long selEnd, long xCurrent, long yCurrent, Boolean reload,
								unsigned waveNum, unsigned callbackType )
{
	if ( (*wavePak)[waveNum-1].fLock == ACTIVE )
	{
		(*wavePak)[waveNum-1].graphCmd->redisplay (
					(*wavePak)[waveNum-1].startRange,
					(*wavePak)[waveNum-1].endRange,
					(*wavePak)[waveNum-1].startSel,
					(*wavePak)[waveNum-1].endSel,
					(*wavePak)[waveNum-1].xCurrent,
					(*wavePak)[waveNum-1].dataOffset );
		return;
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize;

	(*wavePak)[waveNum-1].startRange = start;
	(*wavePak)[waveNum-1].endRange = end;
	(*wavePak)[waveNum-1].startSel = selStart;
	(*wavePak)[waveNum-1].endSel = selEnd;
	(*wavePak)[waveNum-1].xCurrent = xCurrent;
	(*wavePak)[waveNum-1].yCurrent = yCurrent;

	if ( callbackType != 0 )	// re-enable view popup on 'ok' or 'cancel'
	{
		(*wavePak)[waveNum-1].intArraySize = end - start + 1;
		(*wavePak)[waveNum-1].graphNum = 0;

		// if the pointer's non-null the time data view hasn't been cancelled
		if ( (*wavePak)[waveNum-1].waveDataCmd != NULL )
		{
			(*wavePak)[waveNum-1].waveDataCmd->cancel();
			delete (*wavePak)[waveNum-1].waveDataCmd;
			(*wavePak)[waveNum-1].waveDataCmd = NULL;
		}

		(*wavePak)[waveNum-1].dataOffset = (*wavePak)[waveNum-1].startRange;
		delete (*wavePak)[waveNum-1].graphCmd;
		(*wavePak)[waveNum-1].graphCmd = NULL;
	}

	if ( callbackType == 2 )	// clear range and selection on 'Cancel'
	{
		(*wavePak)[waveNum-1].startRange = 0;
		(*wavePak)[waveNum-1].endRange = 0;
		(*wavePak)[waveNum-1].startSel = 0;
		(*wavePak)[waveNum-1].endSel = 0;
		(*wavePak)[waveNum-1].xCurrent = 0;
		(*wavePak)[waveNum-1].yCurrent = 0;
		(*wavePak)[waveNum-1].dataOffset = 0;
		return;
	}
	else
	{
		if ( callbackType == 1 )
			return;	// 'Ok' - just keep pointers to array selection and quit
	}
	
	if ( reload == True )
	{
		(*wavePak)[waveNum-1].waveChanged = False;
		minmax ( start, end, selStart, selEnd );

		(*wavePak)[waveNum-1].pWave->filePos = ( long ) _min * _bytesPerSample;
		blockSize = ( long ) ( ( _max - _min + 1 ) * _bytesPerSample );
		(*wavePak)[waveNum-1].fLock = ACTIVE;
		(*wavePak)[waveNum-1].pWave->LoadBlock ( (*wavePak)[waveNum-1].pWave->filePos, blockSize );
		(*wavePak)[waveNum-1].fLock = INACTIVE;
		createWaveDataArray ( waveNum, ( _max - _min + 1 ) * _bytesPerSample );
		gc->setDataSource ( (*wavePak)[waveNum-1].pIntArray, _max - _min + 1, (*wavePak)[waveNum-1].pWave->totalSamples() );
	}

	// now drive the input to graph-dependent waveDataCmd, if it exists
	if ( (*wavePak)[waveNum-1].waveDataCmd != NULL )
	{
		(*wavePak)[waveNum-1].waveDataCmd->setFields (
						(*wavePak)[waveNum-1].startRange,
						(*wavePak)[waveNum-1].endRange,
						(*wavePak)[waveNum-1].startSel,
						(*wavePak)[waveNum-1].endSel,
						(*wavePak)[waveNum-1].xCurrent,
						(*wavePak)[waveNum-1].yCurrent,
						(*wavePak)[waveNum-1].pWave->totalSamples() );
	}

	// drive parameter views
	if ( _rvbViewRealised == True )
	{
		_spatialPak->rvbView->redisplay (
			( long ) ( (*wavePak)[waveNum-1].startRange * _rvbScale ),
			( long ) ( (*wavePak)[waveNum-1].endRange * _rvbScale ),
			( long ) ( (*wavePak)[waveNum-1].startSel * _rvbScale ),
			( long ) ( (*wavePak)[waveNum-1].endSel * _rvbScale ),
			( long ) ( (*wavePak)[waveNum-1].xCurrent * _rvbScale ),
			( long ) ( (*wavePak)[waveNum-1].dataOffset * _rvbScale ) );
	}

	if ( _rangeViewRealised == True )
	{
		_spatialPak->rangeView->redisplay (
			( long ) ( (*wavePak)[waveNum-1].startRange * _rangeScale ),
			( long ) ( (*wavePak)[waveNum-1].endRange * _rangeScale ),
			( long ) ( (*wavePak)[waveNum-1].startSel * _rangeScale ),
			( long ) ( (*wavePak)[waveNum-1].endSel * _rangeScale ),
			( long ) ( ( *wavePak)[waveNum-1].xCurrent * _rangeScale ),
			( long ) ( (*wavePak)[waveNum-1].dataOffset * _rangeScale ) );
	}

	if ( _bwViewRealised == True )
	{
		_spatialPak->bwView->redisplay (
			( long ) ( (*wavePak)[waveNum-1].startRange * _bwScale ),
			( long ) ( (*wavePak)[waveNum-1].endRange * _bwScale ),
			( long ) ( (*wavePak)[waveNum-1].startSel * _bwScale ),
			( long ) ( (*wavePak)[waveNum-1].endSel * _bwScale ),
			( long ) ( (*wavePak)[waveNum-1].xCurrent * _bwScale ),
			( long ) ( (*wavePak)[waveNum-1].dataOffset * _bwScale ) );
	}

	if ( _sweepViewRealised == True )
	{
		_spatialPak->sweepView->redisplay (
			( long ) ( (*wavePak)[waveNum-1].startRange * _sweepScale ),
			( long ) ( (*wavePak)[waveNum-1].endRange * _sweepScale ),
			( long ) ( (*wavePak)[waveNum-1].startSel * _sweepScale ),
			( long ) ( (*wavePak)[waveNum-1].endSel * _sweepScale ),
			( long ) ( (*wavePak)[waveNum-1].xCurrent * _sweepScale ),
			( long ) ( (*wavePak)[waveNum-1].dataOffset * _sweepScale ) );
	}
	if ( _panViewRealised == True )
	{
		_spatialPak->panView->redisplay (
			( long ) ( (*wavePak)[waveNum-1].startRange * _panScale ),
			( long ) ( (*wavePak)[waveNum-1].endRange * _panScale ),
			( long ) ( (*wavePak)[waveNum-1].startSel * _panScale ),
			( long ) ( (*wavePak)[waveNum-1].endSel * _panScale ),
			( long ) ( (*wavePak)[waveNum-1].xCurrent * _panScale ),
			( long ) ( (*wavePak)[waveNum-1].dataOffset * _panScale ) );
	}

	if ( _dopplerViewRealised == True )
	{
		_spatialPak->dopplerView->redisplay (
			( long ) ( (*wavePak)[waveNum-1].startRange * _dplrScale ),
			( long ) ( (*wavePak)[waveNum-1].endRange * _dplrScale ),
			( long ) ( (*wavePak)[waveNum-1].startSel * _dplrScale ),
			( long ) ( (*wavePak)[waveNum-1].endSel * _dplrScale ),
			( long ) ( (*wavePak)[waveNum-1].xCurrent * _dplrScale ),
			( long ) ( (*wavePak)[waveNum-1].dataOffset * _dplrScale ) );
	}

	if ( _rvbLViewRealised == True )
	{
		_spatialPak->rvbLView->redisplay (
			( long ) ( (*wavePak)[waveNum-1].startRange * _rvblScale ),
			( long ) ( (*wavePak)[waveNum-1].endRange * _rvblScale ),
			( long ) ( (*wavePak)[waveNum-1].startSel * _rvblScale ),
			( long ) ( (*wavePak)[waveNum-1].endSel * _rvblScale ),
			( long ) ( (*wavePak)[waveNum-1].xCurrent * _rvblScale ),
			( long ) ( (*wavePak)[waveNum-1].dataOffset * _rvblScale ) );
	}

	if ( _rvbRViewRealised == True )
	{
		_spatialPak->rvbRView->redisplay (
			( long ) ( (*wavePak)[waveNum-1].startRange * _rvbrScale ),
			( long ) ( (*wavePak)[waveNum-1].endRange * _rvbrScale ),
			( long ) ( (*wavePak)[waveNum-1].startSel * _rvbrScale ),
			( long ) ( (*wavePak)[waveNum-1].endSel * _rvbrScale ),
			( long ) ( (*wavePak)[waveNum-1].xCurrent * _rvbrScale ),
			( long ) ( (*wavePak)[waveNum-1].dataOffset * _rvbrScale ) );
	}

	if ( _panLViewRealised == True )
	{
		_spatialPak->panLView->redisplay (
			( long ) ( (*wavePak)[waveNum-1].startRange * _panlScale ),
			( long ) ( (*wavePak)[waveNum-1].endRange * _panlScale ),
			( long ) ( (*wavePak)[waveNum-1].startSel * _panlScale ),
			( long ) ( (*wavePak)[waveNum-1].endSel * _panlScale ),
			( long ) ( (*wavePak)[waveNum-1].xCurrent * _panlScale ),
			( long ) ( (*wavePak)[waveNum-1].dataOffset * _panlScale ) );
	}

	if ( _panRViewRealised == True )
	{
		_spatialPak->panRView->redisplay (
			( long ) ( (*wavePak)[waveNum-1].startRange * _panrScale ),
			( long ) ( (*wavePak)[waveNum-1].endRange * _panrScale ),
			( long ) ( (*wavePak)[waveNum-1].startSel * _panrScale ),
			( long ) ( (*wavePak)[waveNum-1].endSel * _panrScale ),
			( long ) ( (*wavePak)[waveNum-1].xCurrent * _panrScale ),
			( long ) ( (*wavePak)[waveNum-1].dataOffset * _panrScale ) );
	}

	if ( _bwLViewRealised == True )
	{
		_spatialPak->bwLView->redisplay (
			( long ) ( (*wavePak)[waveNum-1].startRange * _bwlScale ),
			( long ) ( (*wavePak)[waveNum-1].endRange * _bwlScale ),
			( long ) ( (*wavePak)[waveNum-1].startSel * _bwlScale ),
			( long ) ( (*wavePak)[waveNum-1].endSel * _bwlScale ),
			( long ) ( (*wavePak)[waveNum-1].xCurrent * _bwlScale ),
			( long ) ( (*wavePak)[waveNum-1].dataOffset * _bwlScale ) );
	}

	if ( _bwRViewRealised == True )
	{
		_spatialPak->bwRView->redisplay (
			( long ) ( (*wavePak)[waveNum-1].startRange * _bwrScale ),
			( long ) ( (*wavePak)[waveNum-1].endRange * _bwrScale ),
			( long ) ( (*wavePak)[waveNum-1].startSel * _bwrScale ),
			( long ) ( (*wavePak)[waveNum-1].endSel * _bwrScale ),
			( long ) ( (*wavePak)[waveNum-1].xCurrent * _bwrScale ),
			( long ) ( (*wavePak)[waveNum-1].dataOffset * _bwrScale ) );
	}

	if ( _sweepLViewRealised == True )
	{
		_spatialPak->sweepLView->redisplay (
			( long ) ( (*wavePak)[waveNum-1].startRange * _sweeplScale ),
			( long ) ( (*wavePak)[waveNum-1].endRange * _sweeplScale ),
			( long ) ( (*wavePak)[waveNum-1].startSel * _sweeplScale ),
			( long ) ( (*wavePak)[waveNum-1].endSel * _sweeplScale ),
			( long ) ( (*wavePak)[waveNum-1].xCurrent * _sweeplScale ),
			( long ) ( (*wavePak)[waveNum-1].dataOffset * _sweeplScale ) );
	}

	if ( _sweepRViewRealised == True )
	{
		_spatialPak->sweepRView->redisplay (
			( long ) ( (*wavePak)[waveNum-1].startRange * _sweeprScale ),
			( long ) ( (*wavePak)[waveNum-1].endRange * _sweeprScale ),
			( long ) ( (*wavePak)[waveNum-1].startSel * _sweeprScale ),
			( long ) ( (*wavePak)[waveNum-1].endSel * _sweeprScale ),
			( long ) ( (*wavePak)[waveNum-1].xCurrent * _sweeprScale ),
			( long ) ( (*wavePak)[waveNum-1].dataOffset * _sweeprScale ) );
	}

	if ( _ampLViewRealised == True )
	{
		_spatialPak->ampLView->redisplay (
			( long ) ( (*wavePak)[waveNum-1].startRange * _amplScale ),
			( long ) ( (*wavePak)[waveNum-1].endRange * _amplScale ),
			( long ) ( (*wavePak)[waveNum-1].startSel * _amplScale ),
			( long ) ( (*wavePak)[waveNum-1].endSel * _amplScale ),
			( long ) ( (*wavePak)[waveNum-1].xCurrent * _amplScale ),
			( long ) ( (*wavePak)[waveNum-1].dataOffset * _amplScale ) );
	}

	if ( _ampRViewRealised == True )
	{
		_spatialPak->ampRView->redisplay (
			( long ) ( (*wavePak)[waveNum-1].startRange * _amprScale ),
			( long ) ( (*wavePak)[waveNum-1].endRange * _amprScale ),
			( long ) ( (*wavePak)[waveNum-1].startSel * _amprScale ),
			( long ) ( (*wavePak)[waveNum-1].endSel * _amprScale ),
			( long ) ( (*wavePak)[waveNum-1].xCurrent * _amprScale ),
			( long ) ( (*wavePak)[waveNum-1].dataOffset * _amprScale ) );
	}
}


void CreateSpatialTemplateCmd::createWaveDataArray ( unsigned waveNum, long blockSize )
{
	char *pData;
	unsigned int i, j;
	int arrayVal;

	delete[] ( *wavePak )[ waveNum - 1 ].pIntArray;	// delete original array pointer
	pData = (*wavePak)[waveNum-1].pWave->data();

	if (((*wavePak)[waveNum-1].pWave->dataLength() < blockSize) || (blockSize == 0))
		_currentSamples = (*wavePak)[waveNum-1].pWave->totalSamples();
	else
		_currentSamples = blockSize / _bytesPerSample;

	// loaded block size is preserved so wave editor can be re-popped up after 'ok' 
	// or 'cancel', avoiding wave reload with default mixwaveResources->blockLength()
	(*wavePak)[waveNum-1].intArraySize = _currentSamples;
	(*wavePak)[waveNum-1].pIntArray = new int [_currentSamples];

	// convert pWave data to signed 16-bit format
	for ( long currentSample = 0; currentSample < _currentSamples; currentSample++ )
	{
		i = (unsigned char) pData[currentSample*2];
		j = (unsigned char) pData[currentSample*2+1];

		arrayVal = j * 256 + i;

		if ( arrayVal > mixwaveResources->maxY() )
			arrayVal -= ( mixwaveResources->maxY() - mixwaveResources->minY() + 1);

		(*wavePak)[waveNum-1].pIntArray[currentSample] = arrayVal;
	}
}


void CreateSpatialTemplateCmd::viewReverbTemplateCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, float yCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	GraphFltCmd *gfc = ( GraphFltCmd * ) cmd;

	obj->viewReverbTemplateCmdCallbackFinished ( NULL, gfc, start, end, selStart, selEnd,
											xCurrent, 0, yCurrent, reload, templateNum,
											callbackType );
}


void CreateSpatialTemplateCmd::viewReverbTemplateCmdCallbackFinished (
			GraphIntCmd *gic, GraphFltCmd *gfc, long start, long end,
			long selStart, long selEnd, long xCurrent,
			long yIntCurrent, float yFltCurrent, Boolean reload,
			unsigned templateNum, unsigned callbackType )
{
	if ( _spatialPak->rvbFLock == ACTIVE )
	{
		_spatialPak->rvbView->redisplay (
				_spatialPak->rvbStartRange,
				_spatialPak->rvbEndRange,
				_spatialPak->rvbStartSel,
				_spatialPak->rvbEndSel,
				_spatialPak->rvbXCurrent,
				_spatialPak->rvbDataOffset );		
		return;
	}

	if ( ( _spatialPak->rvbView->dataChanged() != 0 ) || ( _shapeCmd->dataChanged() != 0  ) )
	{
		int offset = _spatialPak->rvbView->startData();
		offset *= sizeof ( float );

		// since file length is unaltered, just contents, write the block out
		if ( _spatialPak->rvbFLock == INACTIVE )
		{
			_spatialPak->rvbFLock = ACTIVE;
			FileAccess *out = new FileAccess ( _rvbFileName, "r+" );

			out->filePos ( offset );
			out->writeF ( _spatialPak->rvbArray, sizeof ( float ), _spatialPak->rvbArraySize );

			delete out;
			_spatialPak->rvbFLock = INACTIVE;			

			if ( _shapeCmd->dataChanged() != 0 ) 
				_shapeCmd->reset();
		}
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize;

	_spatialPak->rvbStartRange = start;
	_spatialPak->rvbEndRange = end;
	_spatialPak->rvbStartSel = selStart;
	_spatialPak->rvbEndSel = selEnd;
	_spatialPak->rvbXCurrent = xCurrent;
	_spatialPak->rvbYCurrent = yFltCurrent;

	if ( reload == True )
	{
		minmax ( start, end, selStart, selEnd );
		_spatialPak->rvbFilePos = _min * sizeof ( float );
		blockSize = ( _max - _min + 1 ) * sizeof ( float );
		_spatialPak->rvbArraySize = ( int ) ( _max - _min + 1 );
		// use dataOffset to indicate datum in file referenced by array[0]
		_spatialPak->rvbDataOffset = _spatialPak->rvbFilePos / sizeof ( float );
		_spatialPak->rvbArray = createFloatDataArray ( _rvbFileName, _spatialPak->rvbArray, blockSize, _spatialPak->rvbFilePos );
		gfc->setDataSource ( _spatialPak->rvbArray, _max - _min + 1, _spatialPak->rvbDataSize );
	}

	_shapeCmd->updateSelection ( templateNum, start, end, selStart, selEnd,
									_spatialPak->rvbDataOffset, NULL, _spatialPak->rvbArray );
}


void CreateSpatialTemplateCmd::viewRangeTemplateCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, float yCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
 	// valid callback types: '0' report, '1' ok, '2' cancel.
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	GraphFltCmd *gfc = ( GraphFltCmd * ) cmd;

	obj->viewRangeTemplateCmdCallbackFinished ( NULL, gfc, start, end, selStart, selEnd,
											xCurrent, 0, yCurrent, reload, templateNum,
											callbackType );
}


void CreateSpatialTemplateCmd::viewRangeTemplateCmdCallbackFinished (
			GraphIntCmd *gic, GraphFltCmd *gfc, long start, long end,
			long selStart, long selEnd, long xCurrent,
			long yIntCurrent, float yFltCurrent, Boolean reload,
			unsigned templateNum, unsigned callbackType )
{
	if ( _spatialPak->rangeFLock == ACTIVE )
	{
		_spatialPak->rangeView->redisplay (
			_spatialPak->rangeStartRange,
			_spatialPak->rangeEndRange,
			_spatialPak->rangeStartSel,
			_spatialPak->rangeEndSel,
			_spatialPak->rangeXCurrent,
			_spatialPak->rangeDataOffset );		
		return;
	}

	if ( ( _spatialPak->rangeView->dataChanged() != 0 ) || ( _shapeCmd->dataChanged() != 0  ) )
	{
		int offset = _spatialPak->rangeView->startData();
		offset *= sizeof ( float );

		// since file length is unaltered, just contents, write the block out
		if ( _spatialPak->rangeFLock == INACTIVE )
		{
			_spatialPak->rangeFLock = ACTIVE;
			FileAccess *out = new FileAccess ( _rangeFileName, "r+" );

			out->filePos ( offset );
			out->writeF ( _spatialPak->rangeArray, sizeof ( float ), _spatialPak->rangeArraySize );

			delete out;
			_spatialPak->rangeFLock = INACTIVE;			

			if ( _shapeCmd->dataChanged() != 0 ) 
				_shapeCmd->reset();
		}
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize;

	_spatialPak->rangeStartRange = start;
	_spatialPak->rangeEndRange = end;
	_spatialPak->rangeStartSel = selStart;
	_spatialPak->rangeEndSel = selEnd;
	_spatialPak->rangeXCurrent = xCurrent;
	_spatialPak->rangeYCurrent = yFltCurrent;

	if ( reload == True )
	{
		minmax ( start, end, selStart, selEnd );
		_spatialPak->rangeFilePos = ( long ) ( _min * sizeof ( float ) );
		blockSize = ( long ) ( ( _max - _min + 1 ) * sizeof ( float ) );
		_spatialPak->rangeArraySize = ( int ) ( _max - _min + 1 );
		_spatialPak->rangeDataOffset = _spatialPak->rangeFilePos / sizeof ( float );
		_spatialPak->rangeArray = createFloatDataArray ( _rangeFileName, _spatialPak->rangeArray, blockSize, _spatialPak->rangeFilePos );
		gfc->setDataSource ( _spatialPak->rangeArray, _max - _min + 1, _spatialPak->rangeDataSize );
	}

	_shapeCmd->updateSelection ( templateNum, start, end, selStart, selEnd,
									_spatialPak->rangeDataOffset, NULL, _spatialPak->rangeArray );
}


void CreateSpatialTemplateCmd::viewBwidthTemplateCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, float yCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	GraphFltCmd *gfc = ( GraphFltCmd * ) cmd;

	obj->viewBwidthTemplateCmdCallbackFinished ( NULL, gfc, start, end, selStart, selEnd,
											xCurrent, 0, yCurrent, reload, templateNum,
											callbackType );
}


void CreateSpatialTemplateCmd::viewBwidthTemplateCmdCallbackFinished (
			GraphIntCmd *gic, GraphFltCmd *gfc, long start, long end,
			long selStart, long selEnd, long xCurrent,
			long yIntCurrent, float yFltCurrent, Boolean reload,
			unsigned templateNum, unsigned callbackType )
{
	if ( _spatialPak->bwFLock == ACTIVE )
	{
		_spatialPak->bwView->redisplay (
			_spatialPak->bwStartRange,
			_spatialPak->bwEndRange,
			_spatialPak->bwStartSel,
			_spatialPak->bwEndSel,
			_spatialPak->bwXCurrent,
			_spatialPak->bwDataOffset );		
		return;
	}

	if ( ( _spatialPak->bwView->dataChanged() != 0 ) || ( _shapeCmd->dataChanged() != 0  ) )
	{
		int offset = _spatialPak->bwView->startData();
		offset *= sizeof ( float );

		// since file length is unaltered, just contents, write the block out
		if ( _spatialPak->bwFLock == INACTIVE )
		{
			_spatialPak->bwFLock = ACTIVE;
			FileAccess *out = new FileAccess ( _bwFileName, "r+" );

			out->filePos ( offset );
			out->writeF ( _spatialPak->bwArray, sizeof ( float ), _spatialPak->bwArraySize );

			delete out;
			_spatialPak->bwFLock = INACTIVE;			

			if ( _shapeCmd->dataChanged() != 0 ) 
				_shapeCmd->reset();
		}
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize;

	_spatialPak->bwStartRange = start;
	_spatialPak->bwEndRange = end;
	_spatialPak->bwStartSel = selStart;
	_spatialPak->bwEndSel = selEnd;
	_spatialPak->bwXCurrent = xCurrent;
	_spatialPak->bwYCurrent = yFltCurrent;

	if ( reload == True )
	{
		minmax ( start, end, selStart, selEnd );
		_spatialPak->bwFilePos = ( long ) ( _min * sizeof ( float ) );
		blockSize = ( long ) ( ( _max - _min + 1 ) * sizeof ( float ) );
		_spatialPak->bwArraySize = ( int ) ( _max - _min + 1 );
		_spatialPak->bwDataOffset = _spatialPak->bwFilePos / sizeof ( float );
		_spatialPak->bwArray = createFloatDataArray ( _bwFileName, _spatialPak->bwArray, blockSize, _spatialPak->bwFilePos );
		gfc->setDataSource ( _spatialPak->bwArray, _max - _min + 1, _spatialPak->bwDataSize );
	}

	_shapeCmd->updateSelection ( templateNum, start, end, selStart, selEnd,
									_spatialPak->bwDataOffset, NULL, _spatialPak->bwArray );
}


void CreateSpatialTemplateCmd::viewSweepTemplateCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, float yCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	GraphFltCmd *gfc = ( GraphFltCmd * ) cmd;

	obj->viewSweepTemplateCmdCallbackFinished ( NULL, gfc, start, end, selStart, selEnd,
							xCurrent, 0, yCurrent, reload, templateNum,
							callbackType );
}


void CreateSpatialTemplateCmd::viewSweepTemplateCmdCallbackFinished (
			GraphIntCmd *gic, GraphFltCmd *gfc, long start, long end,
			long selStart, long selEnd, long xCurrent,
			long yIntCurrent, float yFltCurrent, Boolean reload,
			unsigned templateNum, unsigned callbackType )
{
	if ( _spatialPak->sweepFLock == ACTIVE )
	{
		_spatialPak->sweepView->redisplay (
			_spatialPak->sweepStartRange,
			_spatialPak->sweepEndRange,
			_spatialPak->sweepStartSel,
			_spatialPak->sweepEndSel,
			_spatialPak->sweepXCurrent,
			_spatialPak->sweepDataOffset );		
		return;
	}

	if ( ( _spatialPak->sweepView->dataChanged() != 0 ) || ( _shapeCmd->dataChanged() != 0  ) )
	{
		int offset = _spatialPak->sweepView->startData();
		offset *= sizeof ( float );

		// since file length is unaltered, just contents, write the block out
		if ( _spatialPak->sweepFLock == INACTIVE )
		{
			_spatialPak->sweepFLock = ACTIVE;
			FileAccess *out = new FileAccess ( _sweepFileName, "r+" );

			out->filePos ( offset );
			out->writeF ( _spatialPak->sweepArray, sizeof ( float ), _spatialPak->sweepArraySize );

			delete out;
			_spatialPak->sweepFLock = INACTIVE;			

			if ( _shapeCmd->dataChanged() != 0 ) 
				_shapeCmd->reset();
		}
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize;

	_spatialPak->sweepStartRange = start;
	_spatialPak->sweepEndRange = end;
	_spatialPak->sweepStartSel = selStart;
	_spatialPak->sweepEndSel = selEnd;
	_spatialPak->sweepXCurrent = xCurrent;
	_spatialPak->sweepYCurrent = yFltCurrent;

	if ( reload == True )
{
		minmax ( start, end, selStart, selEnd );
		_spatialPak->sweepFilePos = ( long ) ( _min * sizeof ( float ) );
		blockSize = ( long ) ( ( _max - _min + 1 ) * sizeof ( float ) );
		_spatialPak->sweepArraySize = ( int ) ( _max - _min + 1 );
		_spatialPak->sweepDataOffset = _spatialPak->sweepFilePos / sizeof ( float );
		_spatialPak->sweepArray = createFloatDataArray ( _sweepFileName, _spatialPak->sweepArray, blockSize, _spatialPak->sweepFilePos );
		gfc->setDataSource ( _spatialPak->sweepArray, _max - _min + 1, _spatialPak->sweepDataSize );
	}

	_shapeCmd->updateSelection ( templateNum, start, end, selStart, selEnd,
									_spatialPak->sweepDataOffset, NULL, _spatialPak->sweepArray );
}


void CreateSpatialTemplateCmd::viewPanTemplateCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, float yCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	GraphFltCmd *gfc = ( GraphFltCmd * ) cmd;

	obj->viewPanTemplateCmdCallbackFinished ( NULL, gfc, start, end, selStart, selEnd,
						xCurrent, 0, yCurrent, reload, templateNum,
						callbackType );
}


void CreateSpatialTemplateCmd::viewPanTemplateCmdCallbackFinished (
				GraphIntCmd *gic, GraphFltCmd *gfc, long start, long end,
				long selStart, long selEnd, long xCurrent,
				long yIntCurrent, float yFltCurrent, Boolean reload,
				unsigned templateNum, unsigned callbackType )
{
	if ( _spatialPak->panFLock == ACTIVE )
	{
		_spatialPak->panView->redisplay (
			_spatialPak->panStartRange,
			_spatialPak->panEndRange,
			_spatialPak->panStartSel,
			_spatialPak->panEndSel,
			_spatialPak->panXCurrent,
			_spatialPak->panDataOffset );		
		return;
	}

	if ( ( _spatialPak->panView->dataChanged() != 0 ) || ( _shapeCmd->dataChanged() != 0  ) )
	{
		int offset = _spatialPak->panView->startData();
		offset *= sizeof ( float );

		// since file length is unaltered, just contents, write the block out
		if ( _spatialPak->panFLock == INACTIVE )
		{
			_spatialPak->panFLock = ACTIVE;
			FileAccess *out = new FileAccess ( _panFileName, "r+" );

			out->filePos ( offset );
			out->writeF ( _spatialPak->panArray, sizeof ( float ), _spatialPak->panArraySize );

			delete out;
			_spatialPak->panFLock = INACTIVE;			

			if ( _shapeCmd->dataChanged() != 0 ) 
				_shapeCmd->reset();
		}
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize;

	_spatialPak->panStartRange = start;
	_spatialPak->panEndRange = end;
	_spatialPak->panStartSel = selStart;
	_spatialPak->panEndSel = selEnd;
	_spatialPak->panXCurrent = xCurrent;
	_spatialPak->panYCurrent = yFltCurrent;

	if ( reload == True )
	{
		minmax ( start, end, selStart, selEnd );
		_spatialPak->panFilePos = ( long ) ( _min * sizeof ( float ) );
		blockSize = ( long ) ( ( _max - _min + 1 ) * sizeof ( float ) );
		_spatialPak->panArraySize = ( int ) ( _max - _min + 1 );
		_spatialPak->panDataOffset = _spatialPak->panFilePos / sizeof ( float );
		_spatialPak->panArray = createFloatDataArray ( _panFileName, _spatialPak->panArray, blockSize, _spatialPak->panFilePos );
		gfc->setDataSource ( _spatialPak->panArray, _max - _min + 1, _spatialPak->panDataSize );
	}

	_shapeCmd->updateSelection ( templateNum, start, end, selStart, selEnd,
									_spatialPak->panDataOffset, NULL, _spatialPak->panArray );
}


void CreateSpatialTemplateCmd::viewDopplerTemplateCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, float yCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	GraphFltCmd *gfc = ( GraphFltCmd * ) cmd;

	obj->viewDopplerTemplateCmdCallbackFinished ( NULL, gfc, start, end, selStart, selEnd,
							xCurrent, 0, yCurrent, reload, templateNum,
							callbackType );
}


void CreateSpatialTemplateCmd::viewDopplerTemplateCmdCallbackFinished (
				GraphIntCmd *gic, GraphFltCmd *gfc, long start, long end,
				long selStart, long selEnd, long xCurrent,
				long yIntCurrent, float yFltCurrent, Boolean reload,
				unsigned templateNum, unsigned callbackType )
{
	if ( _spatialPak->dopplerFLock == ACTIVE )
	{
		_spatialPak->dopplerView->redisplay (
			_spatialPak->dopplerStartRange,
			_spatialPak->dopplerEndRange,
			_spatialPak->dopplerStartSel,
			_spatialPak->dopplerEndSel,
			_spatialPak->dopplerXCurrent,
			_spatialPak->dopplerDataOffset );		
		return;
	}

	if ( ( _spatialPak->dopplerView->dataChanged() != 0 ) || ( _shapeCmd->dataChanged() != 0  ) )
	{
		int offset = _spatialPak->dopplerView->startData();
		offset *= sizeof ( float );

		// since file length is unaltered, just contents, write the block out
		if ( _spatialPak->dopplerFLock == INACTIVE )
		{
			_spatialPak->dopplerFLock = ACTIVE;
			FileAccess *out = new FileAccess ( _dplrFileName, "r+" );

			out->filePos ( offset );
			out->writeF ( _spatialPak->dopplerArray, sizeof ( float ), _spatialPak->dopplerArraySize );

			delete out;
			_spatialPak->dopplerFLock = INACTIVE;			

			if ( _shapeCmd->dataChanged() != 0 ) 
				_shapeCmd->reset();
		}
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize;

	_spatialPak->dopplerStartRange = start;
	_spatialPak->dopplerEndRange = end;
	_spatialPak->dopplerStartSel = selStart;
	_spatialPak->dopplerEndSel = selEnd;
	_spatialPak->dopplerXCurrent = xCurrent;
	_spatialPak->dopplerYCurrent = yFltCurrent;

	if ( reload == True )
	{
		minmax ( start, end, selStart, selEnd );
		_spatialPak->dopplerFilePos = ( long ) ( _min * sizeof ( float ) );
		blockSize = ( long ) ( ( _max - _min + 1 ) * sizeof ( float ) );
		_spatialPak->dopplerArraySize = ( int ) ( _max - _min + 1 );
		_spatialPak->dopplerDataOffset = _spatialPak->dopplerFilePos / sizeof ( float );
		_spatialPak->dopplerArray = createFloatDataArray ( _dplrFileName, _spatialPak->dopplerArray, blockSize, _spatialPak->dopplerFilePos );
		gfc->setDataSource ( _spatialPak->dopplerArray, _max - _min + 1, _spatialPak->dopplerDataSize );
	}

	_shapeCmd->updateSelection ( templateNum, start, end, selStart, selEnd,
									_spatialPak->dopplerDataOffset, NULL, _spatialPak->dopplerArray );
}


void CreateSpatialTemplateCmd::viewReverbLTemplateCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, float yCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	GraphFltCmd *gfc = ( GraphFltCmd * ) cmd;

	obj->viewReverbLTemplateCmdCallbackFinished ( NULL, gfc, start, end, selStart, selEnd,
							xCurrent, 0, yCurrent, reload, templateNum,
							callbackType );
}


void CreateSpatialTemplateCmd::viewReverbLTemplateCmdCallbackFinished (
				GraphIntCmd *gic, GraphFltCmd *gfc, long start, long end,
				long selStart, long selEnd, long xCurrent,
				long yIntCurrent, float yFltCurrent, Boolean reload,
				unsigned templateNum, unsigned callbackType )
{
	if ( _spatialPak->rvbLFLock == ACTIVE )
	{
		_spatialPak->rvbLView->redisplay (
			_spatialPak->rvbLStartRange,
			_spatialPak->rvbLEndRange,
			_spatialPak->rvbLStartSel,
			_spatialPak->rvbLEndSel,
			_spatialPak->rvbLXCurrent,
			_spatialPak->rvbLDataOffset );		
		return;
	}

	if ( ( _spatialPak->rvbLView->dataChanged() != 0 ) || ( _shapeCmd->dataChanged() != 0  ) )
	{
		int offset = _spatialPak->rvbLView->startData();
		offset *= sizeof ( float );

		// since file length is unaltered, just contents, write the block out
		if ( _spatialPak->rvbLFLock == INACTIVE )
		{
			_spatialPak->rvbLFLock = ACTIVE;
			FileAccess *out = new FileAccess ( _rvbLFileName, "r+" );

			out->filePos ( offset );
			out->writeF ( _spatialPak->rvbLArray, sizeof ( float ), _spatialPak->rvbLArraySize );

			delete out;
			_spatialPak->rvbLFLock = INACTIVE;			

			if ( _shapeCmd->dataChanged() != 0 ) 
				_shapeCmd->reset();
		}
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize;

	_spatialPak->rvbLStartRange = start;
	_spatialPak->rvbLEndRange = end;
	_spatialPak->rvbLStartSel = selStart;
	_spatialPak->rvbLEndSel = selEnd;
	_spatialPak->rvbLXCurrent = xCurrent;
	_spatialPak->rvbLYCurrent = yFltCurrent;

	if ( reload == True )
	{
		minmax ( start, end, selStart, selEnd );
		_spatialPak->rvbLFilePos = ( long ) ( _min * sizeof ( float ) );
		blockSize = ( long ) ( ( _max - _min + 1 ) * sizeof ( float ) );
		_spatialPak->rvbLArraySize = ( int ) ( _max - _min + 1 );
		_spatialPak->rvbLDataOffset = _spatialPak->rvbLFilePos / sizeof ( float );
		_spatialPak->rvbLArray = createFloatDataArray ( _rvbLFileName, _spatialPak->rvbLArray, blockSize, _spatialPak->rvbLFilePos );
		gfc->setDataSource ( _spatialPak->rvbLArray, _max - _min + 1, _spatialPak->rvbLDataSize );
	}

	_shapeCmd->updateSelection ( templateNum, start, end, selStart, selEnd,
					_spatialPak->rvbLDataOffset, NULL, _spatialPak->rvbLArray );
}


void CreateSpatialTemplateCmd::viewReverbRTemplateCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, float yCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	GraphFltCmd *gfc = ( GraphFltCmd * ) cmd;

	obj->viewReverbRTemplateCmdCallbackFinished ( NULL, gfc, start, end, selStart, selEnd,
							xCurrent, 0, yCurrent, reload, templateNum,
							callbackType );
}


void CreateSpatialTemplateCmd::viewReverbRTemplateCmdCallbackFinished (
				GraphIntCmd *gic, GraphFltCmd *gfc, long start, long end,
				long selStart, long selEnd, long xCurrent,
				long yIntCurrent, float yFltCurrent, Boolean reload,
				unsigned templateNum, unsigned callbackType )
{
	if ( _spatialPak->rvbRFLock == ACTIVE )
	{
		_spatialPak->rvbRView->redisplay (
			_spatialPak->rvbRStartRange,
			_spatialPak->rvbREndRange,
			_spatialPak->rvbRStartSel,
			_spatialPak->rvbREndSel,
			_spatialPak->rvbRXCurrent,
			_spatialPak->rvbRDataOffset );		
		return;
	}

	if ( ( _spatialPak->rvbRView->dataChanged() != 0 ) || ( _shapeCmd->dataChanged() != 0  ) )
	{
		int offset = _spatialPak->rvbRView->startData();
		offset *= sizeof ( float );

		// since file length is unaltered, just contents, write the block out
		if ( _spatialPak->rvbRFLock == INACTIVE )
		{
			_spatialPak->rvbRFLock = ACTIVE;
			FileAccess *out = new FileAccess ( _rvbRFileName, "r+" );

			out->filePos ( offset );
			out->writeF ( _spatialPak->rvbRArray, sizeof ( float ), _spatialPak->rvbRArraySize );

			delete out;
			_spatialPak->rvbRFLock = INACTIVE;			

			if ( _shapeCmd->dataChanged() != 0 ) 
				_shapeCmd->reset();
		}
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize;

	_spatialPak->rvbRStartRange = start;
	_spatialPak->rvbREndRange = end;
	_spatialPak->rvbRStartSel = selStart;
	_spatialPak->rvbREndSel = selEnd;
	_spatialPak->rvbRXCurrent = xCurrent;
	_spatialPak->rvbRYCurrent = yFltCurrent;

	if ( reload == True )
	{
		minmax ( start, end, selStart, selEnd );
		_spatialPak->rvbRFilePos = ( long ) ( _min * sizeof ( float ) );
		blockSize = ( long ) ( ( _max - _min + 1 ) * sizeof ( float ) );
		_spatialPak->rvbRArraySize = ( int ) ( _max - _min + 1 );
		_spatialPak->rvbRDataOffset = _spatialPak->rvbRFilePos / sizeof ( float );
		_spatialPak->rvbRArray = createFloatDataArray ( _rvbRFileName, _spatialPak->rvbRArray, blockSize, _spatialPak->rvbRFilePos );
		gfc->setDataSource ( _spatialPak->rvbRArray, _max - _min + 1, _spatialPak->rvbRDataSize );
	}

	_shapeCmd->updateSelection ( templateNum, start, end, selStart, selEnd,
					_spatialPak->rvbRDataOffset, NULL, _spatialPak->rvbRArray );
}


void CreateSpatialTemplateCmd::viewPanLTemplateCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, float yCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	GraphFltCmd *gfc = ( GraphFltCmd * ) cmd;

	obj->viewPanLTemplateCmdCallbackFinished ( NULL, gfc, start, end, selStart, selEnd,
							xCurrent, 0, yCurrent, reload, templateNum,
							callbackType );
}


void CreateSpatialTemplateCmd::viewPanLTemplateCmdCallbackFinished (
				GraphIntCmd *gic, GraphFltCmd *gfc, long start, long end,
				long selStart, long selEnd, long xCurrent,
				long yIntCurrent, float yFltCurrent, Boolean reload,
				unsigned templateNum, unsigned callbackType )
{
	if ( _spatialPak->panLFLock == ACTIVE )
	{
		_spatialPak->panLView->redisplay (
			_spatialPak->panLStartRange,
			_spatialPak->panLEndRange,
			_spatialPak->panLStartSel,
			_spatialPak->panLEndSel,
			_spatialPak->panLXCurrent,
			_spatialPak->panLDataOffset );		
		return;
	}

	if ( ( _spatialPak->panLView->dataChanged() != 0 ) || ( _shapeCmd->dataChanged() != 0  ) )
	{
		int offset = _spatialPak->panLView->startData();
		offset *= sizeof ( float );

		// since file length is unaltered, just contents, write the block out
		if ( _spatialPak->panLFLock == INACTIVE )
		{
			_spatialPak->panLFLock = ACTIVE;
			FileAccess *out = new FileAccess ( _panLFileName, "r+" );

			out->filePos ( offset );
			out->writeF ( _spatialPak->panLArray, sizeof ( float ), _spatialPak->panLArraySize );

			delete out;
			_spatialPak->panLFLock = INACTIVE;			

			if ( _shapeCmd->dataChanged() != 0 ) 
				_shapeCmd->reset();
		}
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize;

	_spatialPak->panLStartRange = start;
	_spatialPak->panLEndRange = end;
	_spatialPak->panLStartSel = selStart;
	_spatialPak->panLEndSel = selEnd;
	_spatialPak->panLXCurrent = xCurrent;
	_spatialPak->panLYCurrent = yFltCurrent;

	if ( reload == True )
	{
		minmax ( start, end, selStart, selEnd );
		_spatialPak->panLFilePos = ( long ) ( _min * sizeof ( float ) );
		blockSize = ( long ) ( ( _max - _min + 1 ) * sizeof ( float ) );
		_spatialPak->panLArraySize = ( int ) ( _max - _min + 1 );
		_spatialPak->panLDataOffset = _spatialPak->panLFilePos / sizeof ( float );
		_spatialPak->panLArray = createFloatDataArray ( _panLFileName, _spatialPak->panLArray, blockSize, _spatialPak->panLFilePos );
		gfc->setDataSource ( _spatialPak->panLArray, _max - _min + 1, _spatialPak->panLDataSize );
	}

	_shapeCmd->updateSelection ( templateNum, start, end, selStart, selEnd,
					_spatialPak->panLDataOffset, NULL, _spatialPak->panLArray );
}


void CreateSpatialTemplateCmd::viewPanRTemplateCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, float yCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	GraphFltCmd *gfc = ( GraphFltCmd * ) cmd;

	obj->viewPanRTemplateCmdCallbackFinished ( NULL, gfc, start, end, selStart, selEnd,
							xCurrent, 0, yCurrent, reload, templateNum,
							callbackType );
}


void CreateSpatialTemplateCmd::viewPanRTemplateCmdCallbackFinished (
					GraphIntCmd *gic, GraphFltCmd *gfc, long start, long end,
					long selStart, long selEnd, long xCurrent,
					long yIntCurrent, float yFltCurrent, Boolean reload,
					unsigned templateNum, unsigned callbackType )
{
	if ( _spatialPak->panRFLock == ACTIVE )
	{
		_spatialPak->panRView->redisplay (
			_spatialPak->panRStartRange,
			_spatialPak->panREndRange,
			_spatialPak->panRStartSel,
			_spatialPak->panREndSel,
			_spatialPak->panRXCurrent,
			_spatialPak->panRDataOffset );		
		return;
	}

	if ( ( _spatialPak->panRView->dataChanged() != 0 ) || ( _shapeCmd->dataChanged() != 0  ) )
	{
		int offset = _spatialPak->panRView->startData();
		offset *= sizeof ( float );

		// since file length is unaltered, just contents, write the block out
		if ( _spatialPak->panRFLock == INACTIVE )
		{
			_spatialPak->panRFLock = ACTIVE;
			FileAccess *out = new FileAccess ( _panRFileName, "r+" );

			out->filePos ( offset );
			out->writeF ( _spatialPak->panRArray, sizeof ( float ), _spatialPak->panRArraySize );

			delete out;
			_spatialPak->panRFLock = INACTIVE;			

			if ( _shapeCmd->dataChanged() != 0 ) 
				_shapeCmd->reset();
		}
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize;

	_spatialPak->panRStartRange = start;
	_spatialPak->panREndRange = end;
	_spatialPak->panRStartSel = selStart;
	_spatialPak->panREndSel = selEnd;
	_spatialPak->panRXCurrent = xCurrent;
	_spatialPak->panRYCurrent = yFltCurrent;

	if ( reload == True )
	{
		minmax ( start, end, selStart, selEnd );
		_spatialPak->panRFilePos = ( long ) ( _min * sizeof ( float ) );
		blockSize = ( long ) ( ( _max - _min + 1 ) * sizeof ( float ) );
		_spatialPak->panRArraySize = ( int ) ( _max - _min + 1 );
		_spatialPak->panRDataOffset = _spatialPak->panRFilePos / sizeof ( float );
		_spatialPak->panRArray = createFloatDataArray ( _panRFileName, _spatialPak->panRArray, blockSize, _spatialPak->panRFilePos );
		gfc->setDataSource ( _spatialPak->panRArray, _max - _min + 1, _spatialPak->panRDataSize );
	}

	_shapeCmd->updateSelection ( templateNum, start, end, selStart, selEnd,
					_spatialPak->panRDataOffset, NULL, _spatialPak->panRArray );
}


void CreateSpatialTemplateCmd::viewBwidthLTemplateCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, float yCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	GraphFltCmd *gfc = ( GraphFltCmd * ) cmd;

	obj->viewBwidthLTemplateCmdCallbackFinished ( NULL, gfc, start, end, selStart, selEnd,
							xCurrent, 0, yCurrent, reload, templateNum,
							callbackType );
}


void CreateSpatialTemplateCmd::viewBwidthLTemplateCmdCallbackFinished (
				GraphIntCmd *gic, GraphFltCmd *gfc, long start, long end,
				long selStart, long selEnd, long xCurrent,
				long yIntCurrent, float yFltCurrent, Boolean reload,
				unsigned templateNum, unsigned callbackType )
{
	if ( _spatialPak->bwLFLock == ACTIVE )
	{
		_spatialPak->bwLView->redisplay (
			_spatialPak->bwLStartRange,
			_spatialPak->bwLEndRange,
			_spatialPak->bwLStartSel,
			_spatialPak->bwLEndSel,
			_spatialPak->bwLXCurrent,
			_spatialPak->bwLDataOffset );		
		return;
	}

	if ( ( _spatialPak->bwLView->dataChanged() != 0 ) || ( _shapeCmd->dataChanged() != 0  ) )
	{
		int offset = _spatialPak->bwLView->startData();
		offset *= sizeof ( float );

		// since file length is unaltered, just contents, write the block out
		if ( _spatialPak->bwLFLock == INACTIVE )
		{
			_spatialPak->bwLFLock = ACTIVE;
			FileAccess *out = new FileAccess ( _bwLFileName, "r+" );

			out->filePos ( offset );
			out->writeF ( _spatialPak->bwLArray, sizeof ( float ), _spatialPak->bwLArraySize );

			delete out;
			_spatialPak->bwLFLock = INACTIVE;			

			if ( _shapeCmd->dataChanged() != 0 ) 
				_shapeCmd->reset();
		}
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize;

	_spatialPak->bwLStartRange = start;
	_spatialPak->bwLEndRange = end;
	_spatialPak->bwLStartSel = selStart;
	_spatialPak->bwLEndSel = selEnd;
	_spatialPak->bwLXCurrent = xCurrent;
	_spatialPak->bwLYCurrent = yFltCurrent;

	if ( reload == True )
	{
		minmax ( start, end, selStart, selEnd );
		_spatialPak->bwLFilePos = ( long ) ( _min * sizeof ( float ) );
		blockSize = ( long ) ( ( _max - _min + 1 ) * sizeof ( float ) );
		_spatialPak->bwLArraySize = ( int ) ( _max - _min + 1 );
		_spatialPak->bwLDataOffset = _spatialPak->bwLFilePos / sizeof ( float );
		_spatialPak->bwLArray = createFloatDataArray ( _bwLFileName, _spatialPak->bwLArray, blockSize, _spatialPak->bwLFilePos );
		gfc->setDataSource ( _spatialPak->bwLArray, _max - _min + 1, _spatialPak->bwLDataSize );
	}

	_shapeCmd->updateSelection ( templateNum, start, end, selStart, selEnd,
					_spatialPak->bwLDataOffset, NULL, _spatialPak->bwLArray );
}


void CreateSpatialTemplateCmd::viewBwidthRTemplateCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, float yCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	GraphFltCmd *gfc = ( GraphFltCmd * ) cmd;

	obj->viewBwidthRTemplateCmdCallbackFinished ( NULL, gfc, start, end, selStart, selEnd,
							xCurrent, 0, yCurrent, reload, templateNum,
							callbackType );
}


void CreateSpatialTemplateCmd::viewBwidthRTemplateCmdCallbackFinished (
				GraphIntCmd *gic, GraphFltCmd *gfc, long start, long end,
				long selStart, long selEnd, long xCurrent,
				long yIntCurrent, float yFltCurrent, Boolean reload,
				unsigned templateNum, unsigned callbackType )
{
	if ( _spatialPak->bwRFLock == ACTIVE )
	{
		_spatialPak->bwRView->redisplay (
			_spatialPak->bwRStartRange,
			_spatialPak->bwREndRange,
			_spatialPak->bwRStartSel,
			_spatialPak->bwREndSel,
			_spatialPak->bwRXCurrent,
			_spatialPak->bwRDataOffset );		
		return;
	}

	if ( ( _spatialPak->bwRView->dataChanged() != 0 ) || ( _shapeCmd->dataChanged() != 0  ) )
	{
		int offset = _spatialPak->bwRView->startData();
		offset *= sizeof ( float );

		// since file length is unaltered, just contents, write the block out
		if ( _spatialPak->bwRFLock == INACTIVE )
		{
			_spatialPak->bwRFLock = ACTIVE;
			FileAccess *out = new FileAccess ( _bwRFileName, "r+" );

			out->filePos ( offset );
			out->writeF ( _spatialPak->bwRArray, sizeof ( float ), _spatialPak->bwRArraySize );

			delete out;
			_spatialPak->bwRFLock = INACTIVE;			

			if ( _shapeCmd->dataChanged() != 0 ) 
				_shapeCmd->reset();
		}
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize;

	_spatialPak->bwRStartRange = start;
	_spatialPak->bwREndRange = end;
	_spatialPak->bwRStartSel = selStart;
	_spatialPak->bwREndSel = selEnd;
	_spatialPak->bwRXCurrent = xCurrent;
	_spatialPak->bwRYCurrent = yFltCurrent;

	if ( reload == True )
	{
		minmax ( start, end, selStart, selEnd );
		_spatialPak->bwRFilePos = ( long ) ( _min * sizeof ( float ) );
		blockSize = ( long ) ( ( _max - _min + 1 ) * sizeof ( float ) );
		_spatialPak->bwRArraySize = ( int ) ( _max - _min + 1 );
		_spatialPak->bwRDataOffset = _spatialPak->bwRFilePos / sizeof ( float );
		_spatialPak->bwRArray = createFloatDataArray ( _bwRFileName, _spatialPak->bwRArray, blockSize, _spatialPak->bwRFilePos );
		gfc->setDataSource ( _spatialPak->bwRArray, _max - _min + 1, _spatialPak->bwRDataSize );
	}

	_shapeCmd->updateSelection ( templateNum, start, end, selStart, selEnd,
					_spatialPak->bwRDataOffset, NULL, _spatialPak->bwRArray );
}


void CreateSpatialTemplateCmd::viewSweepLTemplateCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, float yCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	GraphFltCmd *gfc = ( GraphFltCmd * ) cmd;

	obj->viewSweepLTemplateCmdCallbackFinished ( NULL, gfc, start, end, selStart, selEnd,
							xCurrent, 0, yCurrent, reload, templateNum,
							callbackType );
}


void CreateSpatialTemplateCmd::viewSweepLTemplateCmdCallbackFinished (
				GraphIntCmd *gic, GraphFltCmd *gfc, long start, long end,
				long selStart, long selEnd, long xCurrent,
				long yIntCurrent, float yFltCurrent, Boolean reload,
				unsigned templateNum, unsigned callbackType )
{
	if ( _spatialPak->sweepLFLock == ACTIVE )
	{
		_spatialPak->sweepLView->redisplay (
			_spatialPak->sweepLStartRange,
			_spatialPak->sweepLEndRange,
			_spatialPak->sweepLStartSel,
			_spatialPak->sweepLEndSel,
			_spatialPak->sweepLXCurrent,
			_spatialPak->sweepLDataOffset );		
		return;
	}

	if ( ( _spatialPak->sweepLView->dataChanged() != 0 ) || ( _shapeCmd->dataChanged() != 0  ) )
	{
		int offset = _spatialPak->sweepLView->startData();
		offset *= sizeof ( float );

		// since file length is unaltered, just contents, write the block out
		if ( _spatialPak->sweepLFLock == INACTIVE )
		{
			_spatialPak->sweepLFLock = ACTIVE;
			FileAccess *out = new FileAccess ( _sweepLFileName, "r+" );

			out->filePos ( offset );
			out->writeF ( _spatialPak->sweepLArray, sizeof ( float ), _spatialPak->sweepLArraySize );

			delete out;
			_spatialPak->sweepLFLock = INACTIVE;			

			if ( _shapeCmd->dataChanged() != 0 ) 
				_shapeCmd->reset();
		}
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize;

	_spatialPak->sweepLStartRange = start;
	_spatialPak->sweepLEndRange = end;
	_spatialPak->sweepLStartSel = selStart;
	_spatialPak->sweepLEndSel = selEnd;
	_spatialPak->sweepLXCurrent = xCurrent;
	_spatialPak->sweepLYCurrent = yFltCurrent;

	if ( reload == True )
	{
		minmax ( start, end, selStart, selEnd );
		_spatialPak->sweepLFilePos = ( long ) ( _min * sizeof ( float ) );
		blockSize = ( long ) ( ( _max - _min + 1 ) * sizeof ( float ) );
		_spatialPak->sweepLArraySize = ( int ) ( _max - _min + 1 );
		_spatialPak->sweepLDataOffset = _spatialPak->sweepLFilePos / sizeof ( float );
		_spatialPak->sweepLArray = createFloatDataArray ( _sweepLFileName, _spatialPak->sweepLArray, blockSize, _spatialPak->sweepLFilePos );
		gfc->setDataSource ( _spatialPak->sweepLArray, _max - _min + 1, _spatialPak->sweepLDataSize );
	}

	_shapeCmd->updateSelection ( templateNum, start, end, selStart, selEnd,
					_spatialPak->sweepLDataOffset, NULL, _spatialPak->sweepLArray );
}


void CreateSpatialTemplateCmd::viewSweepRTemplateCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, float yCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	GraphFltCmd *gfc = ( GraphFltCmd * ) cmd;

	obj->viewSweepRTemplateCmdCallbackFinished ( NULL, gfc, start, end, selStart, selEnd,
							xCurrent, 0, yCurrent, reload, templateNum,
							callbackType );
}


void CreateSpatialTemplateCmd::viewSweepRTemplateCmdCallbackFinished (
				GraphIntCmd *gic, GraphFltCmd *gfc, long start, long end,
				long selStart, long selEnd, long xCurrent,
				long yIntCurrent, float yFltCurrent, Boolean reload,
				unsigned templateNum, unsigned callbackType )
{
	if ( _spatialPak->sweepRFLock == ACTIVE )
	{
		_spatialPak->sweepRView->redisplay (
			_spatialPak->sweepRStartRange,
			_spatialPak->sweepREndRange,
			_spatialPak->sweepRStartSel,
			_spatialPak->sweepREndSel,
			_spatialPak->sweepRXCurrent,
			_spatialPak->sweepRDataOffset );		
		return;
	}

	if ( ( _spatialPak->sweepRView->dataChanged() != 0 ) || ( _shapeCmd->dataChanged() != 0  ) )
	{
		int offset = _spatialPak->sweepRView->startData();
		offset *= sizeof ( float );

		// since file length is unaltered, just contents, write the block out
		if ( _spatialPak->sweepRFLock == INACTIVE )
		{
			_spatialPak->sweepRFLock = ACTIVE;
			FileAccess *out = new FileAccess ( _sweepRFileName, "r+" );

			out->filePos ( offset );
			out->writeF ( _spatialPak->sweepRArray, sizeof ( float ), _spatialPak->sweepRArraySize );

			delete out;
			_spatialPak->sweepRFLock = INACTIVE;			

			if ( _shapeCmd->dataChanged() != 0 ) 
				_shapeCmd->reset();
		}
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize;

	_spatialPak->sweepRStartRange = start;
	_spatialPak->sweepREndRange = end;
	_spatialPak->sweepRStartSel = selStart;
	_spatialPak->sweepREndSel = selEnd;
	_spatialPak->sweepRXCurrent = xCurrent;
	_spatialPak->sweepRYCurrent = yFltCurrent;

	if ( reload == True )
	{
		minmax ( start, end, selStart, selEnd );
		_spatialPak->sweepRFilePos = ( long ) ( _min * sizeof ( float ) );
		blockSize = ( long ) ( ( _max - _min + 1 ) * sizeof ( float ) );
		_spatialPak->sweepRArraySize = ( int ) ( _max - _min + 1 );
		_spatialPak->sweepRDataOffset = _spatialPak->sweepRFilePos / sizeof ( float );
		_spatialPak->sweepRArray = createFloatDataArray ( _sweepRFileName, _spatialPak->sweepRArray, blockSize, _spatialPak->sweepRFilePos );
		gfc->setDataSource ( _spatialPak->sweepRArray, _max - _min + 1, _spatialPak->sweepRDataSize );
	}

	_shapeCmd->updateSelection ( templateNum, start, end, selStart, selEnd,
					_spatialPak->sweepRDataOffset, NULL, _spatialPak->sweepRArray );
}


void CreateSpatialTemplateCmd::viewAmpLTemplateCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, float yCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	GraphFltCmd *gfc = ( GraphFltCmd * ) cmd;

	obj->viewAmpLTemplateCmdCallbackFinished ( NULL, gfc, start, end, selStart, selEnd,
							xCurrent, 0, yCurrent, reload, templateNum,
							callbackType );
}


void CreateSpatialTemplateCmd::viewAmpLTemplateCmdCallbackFinished (
				GraphIntCmd *gic, GraphFltCmd *gfc, long start, long end,
				long selStart, long selEnd, long xCurrent,
				long yIntCurrent, float yFltCurrent, Boolean reload,
				unsigned templateNum, unsigned callbackType )
{
	if ( _spatialPak->ampLFLock == ACTIVE )
	{
		_spatialPak->ampLView->redisplay (
			_spatialPak->ampLStartRange,
			_spatialPak->ampLEndRange,
			_spatialPak->ampLStartSel,
			_spatialPak->ampLEndSel,
			_spatialPak->ampLXCurrent,
			_spatialPak->ampLDataOffset );		
		return;
	}

	if ( ( _spatialPak->ampLView->dataChanged() != 0 ) || ( _shapeCmd->dataChanged() != 0  ) )
	{
		int offset = _spatialPak->ampLView->startData();
		offset *= sizeof ( float );

		// since file length is unaltered, just contents, write the block out
		if ( _spatialPak->ampLFLock == INACTIVE )
		{
			_spatialPak->ampLFLock = ACTIVE;
			FileAccess *out = new FileAccess ( _ampLFileName, "r+" );

			out->filePos ( offset );
			out->writeF ( _spatialPak->ampLArray, sizeof ( float ), _spatialPak->ampLArraySize );

			delete out;
			_spatialPak->ampLFLock = INACTIVE;			

			if ( _shapeCmd->dataChanged() != 0 ) 
				_shapeCmd->reset();
		}
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize;

	_spatialPak->ampLStartRange = start;
	_spatialPak->ampLEndRange = end;
	_spatialPak->ampLStartSel = selStart;
	_spatialPak->ampLEndSel = selEnd;
	_spatialPak->ampLXCurrent = xCurrent;
	_spatialPak->ampLYCurrent = yFltCurrent;

	if ( reload == True )
	{
		minmax ( start, end, selStart, selEnd );
		_spatialPak->ampLFilePos = ( long ) ( _min * sizeof ( float ) );
		blockSize = ( long ) ( ( _max - _min + 1 ) * sizeof ( float ) );
		_spatialPak->ampLArraySize = ( int ) ( _max - _min + 1 );
		_spatialPak->ampLDataOffset = _spatialPak->ampLFilePos / sizeof ( float );
		_spatialPak->ampLArray = createFloatDataArray ( _ampLFileName, _spatialPak->ampLArray, blockSize, _spatialPak->ampLFilePos );
		gfc->setDataSource ( _spatialPak->ampLArray, _max - _min + 1, _spatialPak->ampLDataSize );
	}

	_shapeCmd->updateSelection ( templateNum, start, end, selStart, selEnd,
					_spatialPak->ampLDataOffset, NULL, _spatialPak->ampLArray );
}


void CreateSpatialTemplateCmd::viewAmpRTemplateCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, float yCurrent, Boolean reload,
								unsigned templateNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	GraphFltCmd *gfc = ( GraphFltCmd * ) cmd;

	obj->viewAmpRTemplateCmdCallbackFinished ( NULL, gfc, start, end, selStart, selEnd,
							xCurrent, 0, yCurrent, reload, templateNum,
							callbackType );
}


void CreateSpatialTemplateCmd::viewAmpRTemplateCmdCallbackFinished (
				GraphIntCmd *gic, GraphFltCmd *gfc, long start, long end,
				long selStart, long selEnd, long xCurrent,
				long yIntCurrent, float yFltCurrent, Boolean reload,
				unsigned templateNum, unsigned callbackType )
{
	if ( _spatialPak->ampRFLock == ACTIVE )
	{
		_spatialPak->ampRView->redisplay (
			_spatialPak->ampRStartRange,
			_spatialPak->ampREndRange,
			_spatialPak->ampRStartSel,
			_spatialPak->ampREndSel,
			_spatialPak->ampRXCurrent,
			_spatialPak->ampRDataOffset );		
		return;
	}

	if ( ( _spatialPak->ampRView->dataChanged() != 0 ) || ( _shapeCmd->dataChanged() != 0  ) )
	{
		int offset = _spatialPak->ampRView->startData();
		offset *= sizeof ( float );

		// since file length is unaltered, just contents, write the block out
		if ( _spatialPak->ampRFLock == INACTIVE )
		{
			_spatialPak->ampRFLock = ACTIVE;
			FileAccess *out = new FileAccess ( _ampRFileName, "r+" );

			out->filePos ( offset );
			out->writeF ( _spatialPak->ampRArray, sizeof ( float ), _spatialPak->ampRArraySize );

			delete out;
			_spatialPak->ampRFLock = INACTIVE;			

			if ( _shapeCmd->dataChanged() != 0 ) 
				_shapeCmd->reset();
		}
	}

	// 'output' used to drive 'inputs' to data objects
	long blockSize;

	_spatialPak->ampRStartRange = start;
	_spatialPak->ampREndRange = end;
	_spatialPak->ampRStartSel = selStart;
	_spatialPak->ampREndSel = selEnd;
	_spatialPak->ampRXCurrent = xCurrent;
	_spatialPak->ampRYCurrent = yFltCurrent;

	if ( reload == True )
	{
		minmax ( start, end, selStart, selEnd );
		_spatialPak->ampRFilePos = ( long ) ( _min * sizeof ( float ) );
		blockSize = ( long ) ( ( _max - _min + 1 ) * sizeof ( float ) );
		_spatialPak->ampRArraySize = ( int ) ( _max - _min + 1 );
		_spatialPak->ampRDataOffset = _spatialPak->ampRFilePos / sizeof ( float );
		_spatialPak->ampRArray = createFloatDataArray ( _ampRFileName, _spatialPak->ampRArray, blockSize, _spatialPak->ampRFilePos );
		gfc->setDataSource ( _spatialPak->ampRArray, _max - _min + 1, _spatialPak->ampRDataSize );
	}

	_shapeCmd->updateSelection ( templateNum, start, end, selStart, selEnd,
					_spatialPak->ampRDataOffset, NULL, _spatialPak->ampRArray );
}


void CreateSpatialTemplateCmd::panTypeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->panTypeEntered ( text );

	obj->_panTypeEditRealised = False;
}


void CreateSpatialTemplateCmd::panTypeEntered ( char *text )
{
	unsigned int num;
	sscanf ( text, "%u", &num );

	if ( ( num >= 0 ) && ( num < 3 ) )	// 3 pan types
		_spatialPak->panType = num;
}


void CreateSpatialTemplateCmd::panTypeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_panTypeEditRealised = False;
}


void CreateSpatialTemplateCmd::filterTypeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->filterTypeEntered ( text );

	obj->_filterTypeEditRealised = False;
}


void CreateSpatialTemplateCmd::filterTypeEntered ( char *text )
{
	int num = -1;
	
	if ( ( strncmp ( text, "band", 4 ) == 0 ) || ( strncmp ( text, "Band", 4 ) == 0 ) )
		num = 0;
	
	if ( ( strncmp ( text, "notch", 4 ) == 0 ) || ( strncmp ( text, "Notch", 4 ) == 0 ) )
		num = 1;
		
	if ( num == -1 )
		sscanf ( text, "%d", &num );

	if ( ( num == 0 ) || ( num == 1 ) )	// band / notch
		_spatialPak->filterType = ( unsigned int ) num;
}


void CreateSpatialTemplateCmd::filterTypeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_filterTypeEditRealised = False;
}


void CreateSpatialTemplateCmd::angularSepEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->angularSepEntered ( text );

	obj->_angularSepEditRealised = False;
}


void CreateSpatialTemplateCmd::angularSepEntered ( char *text )
{
	float num;
	sscanf ( text, "%f", &num );

	if ( ( num >= 0 ) && ( num <= 1 ) )
		_spatialPak->angularSep = num;
}


void CreateSpatialTemplateCmd::angularSepCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_angularSepEditRealised = False;
}


void CreateSpatialTemplateCmd::signalScaleEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->signalScaleEntered ( text );

	obj->_signalScaleEditRealised = False;
}


void CreateSpatialTemplateCmd::signalScaleEntered ( char *text )
{
	float num;
	sscanf ( text, "%f", &num );

	if ( num >= 0 )
		_spatialPak->signalScale = num;
}


void CreateSpatialTemplateCmd::signalScaleCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_signalScaleEditRealised = False;
}


void CreateSpatialTemplateCmd::detuneEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->detuneEntered ( text );

	obj->_detuneEditRealised = False;
}


void CreateSpatialTemplateCmd::detuneEntered ( char *text )
{
	float num;
	sscanf ( text, "%f", &num );

	if ( num > 0 )
		_spatialPak->detune = num;
}


void CreateSpatialTemplateCmd::detuneCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_detuneEditRealised = False;
}


void CreateSpatialTemplateCmd::phaserEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->phaserEntered ( text );

	obj->_phaserEditRealised = False;
}


void CreateSpatialTemplateCmd::phaserEntered ( char *text )
{
	int num = -1;

	if ( ( strcmp ( text, "On" ) == 0 ) || ( strcmp ( text, "on" ) == 0 ) || ( strcmp ( text, "ON" ) == 0 ) )
		num = 1;

	if ( ( strcmp ( text, "True" ) == 0 ) || ( strcmp ( text, "true" ) == 0 ) || ( strcmp ( text, "TRUE" ) == 0 ) )
		num = 1;
	
	if ( ( strcmp ( text, "Off" ) == 0 ) || ( strcmp ( text, "off" ) == 0 ) || ( strcmp ( text, "OFF" ) == 0 ) )
		num = 0;

	if ( ( strcmp ( text, "False" ) == 0 ) || ( strcmp ( text, "false" ) == 0 ) || ( strcmp ( text, "FALSE" ) == 0 ) )
		num = 0;

	if ( num == -1 )	// numeric value entered
		sscanf ( text, "%d", &num );

	if ( ( num == 0 ) || ( num == 1 ) )
		_spatialPak->phaser = ( unsigned int ) num;		
}


void CreateSpatialTemplateCmd::phaserCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_phaserEditRealised = False;
}


void CreateSpatialTemplateCmd::rvbTailEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->rvbTailEntered ( text );

	obj->_rvbTailEditRealised = False;
}


void CreateSpatialTemplateCmd::rvbTailEntered ( char *text )
{
	float num;
	sscanf ( text, "%f", &num );

	if ( num >= 0 )
		_spatialPak->rvbTailTime = num;
}


void CreateSpatialTemplateCmd::rvbTailCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_rvbTailEditRealised = False;
}


void CreateSpatialTemplateCmd::panLTypeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->panLTypeEntered ( text );

	obj->_panLTypeEditRealised = False;
}


void CreateSpatialTemplateCmd::panLTypeEntered ( char *text )
{
	unsigned int num;
	sscanf ( text, "%u", &num );

	if ( ( num == 0 ) || ( num == 1 ) )	// 2 panL types
		_spatialPak->panLType = num;
}


void CreateSpatialTemplateCmd::panLTypeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_panLTypeEditRealised = False;
}


void CreateSpatialTemplateCmd::panRTypeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->panRTypeEntered ( text );

	obj->_panRTypeEditRealised = False;
}


void CreateSpatialTemplateCmd::panRTypeEntered ( char *text )
{
	unsigned int num;
	sscanf ( text, "%u", &num );

	if ( ( num == 0 ) || ( num == 1 ) )	// 2 panR types
		_spatialPak->panRType = num;
}


void CreateSpatialTemplateCmd::panRTypeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_panRTypeEditRealised = False;
}


void CreateSpatialTemplateCmd::delayLEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->delayLEntered ( text );

	obj->_delayLEditRealised = False;
}


void CreateSpatialTemplateCmd::delayLEntered ( char *text )
{
	unsigned int num;
	sscanf ( text, "%u", &num );

	if ( num >= 0 )	// delay time in seconds
		_spatialPak->delayL = num;
}


void CreateSpatialTemplateCmd::delayLCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_delayLEditRealised = False;
}


void CreateSpatialTemplateCmd::delayREnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->delayREntered ( text );

	obj->_delayREditRealised = False;
}


void CreateSpatialTemplateCmd::delayREntered ( char *text )
{
	unsigned int num;
	sscanf ( text, "%u", &num );

	if (  num >= 0 )
		_spatialPak->delayR = num;
}


void CreateSpatialTemplateCmd::delayRCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_delayREditRealised = False;
}


void CreateSpatialTemplateCmd::filterLTypeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->filterLTypeEntered ( text );

	obj->_filterLTypeEditRealised = False;
}


void CreateSpatialTemplateCmd::filterLTypeEntered ( char *text )
{
	int num = -1;
	
	if ( ( strcmp ( text, "band" ) == 0 ) || ( strcmp ( text, "Band" ) == 0 ) )
		num = 0;
	
	if ( ( strcmp ( text, "notch" ) == 0 ) || ( strcmp ( text, "Notch" ) == 0 ) )
		num = 1;
		
	if ( num == -1 )
		sscanf ( text, "%d", &num );

	if ( ( num == 0 ) || ( num == 2 ) )	// band / notch
		_spatialPak->filterLType = ( unsigned int ) num;
}


void CreateSpatialTemplateCmd::filterLTypeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_filterLTypeEditRealised = False;
}


void CreateSpatialTemplateCmd::filterRTypeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->filterRTypeEntered ( text );

	obj->_filterRTypeEditRealised = False;
}


void CreateSpatialTemplateCmd::filterRTypeEntered ( char *text )
{
	int num = -1;
	
	if ( ( strcmp ( text, "band" ) == 0 ) || ( strcmp ( text, "Band" ) == 0 ) )
		num = 0;
	
	if ( ( strcmp ( text, "notch" ) == 0 ) || ( strcmp ( text, "Notch" ) == 0 ) )
		num = 1;
		
	if ( num == -1 )
		sscanf ( text, "%d", &num );

	if ( ( num == 0 ) || ( num == 2 ) )	// band / notch
		_spatialPak->filterRType = ( unsigned int ) num;
}


void CreateSpatialTemplateCmd::filterRTypeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_filterRTypeEditRealised = False;
}


void CreateSpatialTemplateCmd::rvbRangeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->rvbRangeEntered ( text );

	obj->_rvbRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::rvbRangeEntered ( char *text )
{
	float min, max;
	sscanf ( text, "%f,%f", &min, &max );

	if ( ( min < max ) && ( min >= 0 ) )
	{
		_spatialPak->rvbMin = min;
		_spatialPak->rvbMax = max;
	}
}


void CreateSpatialTemplateCmd::rvbRangeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_rvbRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::distRangeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->distRangeEntered ( text );

	obj->_distRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::distRangeEntered ( char *text )
{
	float min, max;
	sscanf ( text, "%f,%f", &min, &max );

	if ( min < max )	// N.B we allow -ve values here (behaviour unspecified) 
	{
		_spatialPak->rangeMin = min;
		_spatialPak->rangeMax = max;
	}
}


void CreateSpatialTemplateCmd::distRangeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_distRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::bwRangeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->bwRangeEntered ( text );

	obj->_bwRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::bwRangeEntered ( char *text )
{
	float min, max;
	sscanf ( text, "%f,%f", &min, &max );

	if ( ( min < max ) && ( min >= 0 ) )
	{
		_spatialPak->bwMin = min;
		_spatialPak->bwMax = max;
	}
}


void CreateSpatialTemplateCmd::bwRangeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_bwRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::sweepRangeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->sweepRangeEntered ( text );

	obj->_sweepRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::sweepRangeEntered ( char *text )
{
	float min, max;
	sscanf ( text, "%f,%f", &min, &max );

	if ( ( min < max ) && ( min >= 0 ) )
	{
		_spatialPak->sweepMin = min;
		_spatialPak->sweepMax = max;
	}
}


void CreateSpatialTemplateCmd::sweepRangeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_sweepRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::dopplerRangeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->dopplerRangeEntered ( text );

	obj->_dopplerRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::dopplerRangeEntered ( char *text )
{
	float min, max;
	sscanf ( text, "%f,%f", &min, &max );

	if ( ( min < max ) && ( min >= 0 ) )
	{
		_spatialPak->dopplerMin = min;
		_spatialPak->dopplerMax = max;
	}
}


void CreateSpatialTemplateCmd::dopplerRangeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_dopplerRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::rvbLRangeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->rvbLRangeEntered ( text );

	obj->_rvblRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::rvbLRangeEntered ( char *text )
{
	float min, max;
	sscanf ( text, "%f,%f", &min, &max );

	if ( ( min < max ) && ( min >= 0 ) )
	{
		_spatialPak->rvbLMin = min;
		_spatialPak->rvbLMax = max;
	}
}


void CreateSpatialTemplateCmd::rvbLRangeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_rvblRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::rvbRRangeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->rvbRRangeEntered ( text );

	obj->_rvbrRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::rvbRRangeEntered ( char *text )
{
	float min, max;
	sscanf ( text, "%f,%f", &min, &max );

	if ( ( min < max ) && ( min >= 0 ) )
	{
		_spatialPak->rvbRMin = min;
		_spatialPak->rvbRMax = max;
	}
}


void CreateSpatialTemplateCmd::rvbRRangeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_rvbrRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::bwLRangeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->bwLRangeEntered ( text );

	obj->_bwlRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::bwLRangeEntered ( char *text )
{
	float min, max;
	sscanf ( text, "%f,%f", &min, &max );

	if ( ( min < max ) && ( min >= 0 ) )
	{
		_spatialPak->bwLMin = min;
		_spatialPak->bwLMax = max;
	}
}


void CreateSpatialTemplateCmd::bwLRangeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_bwlRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::bwRRangeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->bwRRangeEntered ( text );

	obj->_bwrRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::bwRRangeEntered ( char *text )
{
	float min, max;
	sscanf ( text, "%f,%f", &min, &max );

	if ( ( min < max ) && ( min >= 0 ) )
	{
		_spatialPak->bwRMin = min;
		_spatialPak->bwRMax = max;
	}
}


void CreateSpatialTemplateCmd::bwRRangeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_bwrRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::sweepLRangeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->sweepLRangeEntered ( text );

	obj->_sweeplRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::sweepLRangeEntered ( char *text )
{
	float min, max;
	sscanf ( text, "%f,%f", &min, &max );

	if ( ( min < max ) && ( min >= 0 ) )
	{
		_spatialPak->sweepLMin = min;
		_spatialPak->sweepLMax = max;
	}
}


void CreateSpatialTemplateCmd::sweepLRangeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_sweeplRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::sweepRRangeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->sweepRRangeEntered ( text );

	obj->_sweeprRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::sweepRRangeEntered ( char *text )
{
	float min, max;
	sscanf ( text, "%f,%f", &min, &max );

	if ( ( min < max ) && ( min >= 0 ) )
	{
		_spatialPak->sweepRMin = min;
		_spatialPak->sweepRMax = max;
	}
}


void CreateSpatialTemplateCmd::sweepRRangeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_sweeprRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::ampLRangeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->ampLRangeEntered ( text );

	obj->_amplRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::ampLRangeEntered ( char *text )
{
	float min, max;
	sscanf ( text, "%f,%f", &min, &max );

	if ( ( min < max ) && ( min >= 0 ) )
	{
		_spatialPak->ampLMin = min;
		_spatialPak->ampLMax = max;
	}
}


void CreateSpatialTemplateCmd::ampLRangeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_amplRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::ampRRangeEnteredCallback ( void *clientData, char *text )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( strlen ( text ) > 0 )
		obj->ampRRangeEntered ( text );

	obj->_amprRangeEditRealised = False;
}


void CreateSpatialTemplateCmd::ampRRangeEntered ( char *text )
{
	float min, max;
	sscanf ( text, "%f,%f", &min, &max );

	if ( ( min < max ) && ( min >= 0 ) )
	{
		_spatialPak->ampRMin = min;
		_spatialPak->ampRMax = max;
	}
}


void CreateSpatialTemplateCmd::ampRRangeCancelledCallback ( void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	obj->_rvbTailEditRealised = False;
}


void CreateSpatialTemplateCmd::spatialScoreCreatedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		char msg[28];
		strcpy(msg, "Score creation interrupted." );
		messageDialogManager->post ( msg );
	}
	else		
		obj->spatialScoreCreated ( ( CreateSpatialScoreCmd * ) cmd );
}


void CreateSpatialTemplateCmd::spatialScoreCreated ( CreateSpatialScoreCmd *cssObj )
{
	char msg[BUFSIZE];

	if ( strlen ( cssObj->error() ) != 0 )
	{
		_error = 1;
		snprintf ( msg, sizeof ( msg ), "Error: %s", cssObj->error() );
		msg [ sizeof ( msg ) - 1 ] = 0;
		messageDialogManager->post ( msg );
	}
	else
		_runCsndCmd->execute ( &CreateSpatialTemplateCmd::csoundRunCallback, ( void * ) this );
}


void CreateSpatialTemplateCmd::csoundRunCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;

	if ( interrupted )
	{
		// get pid of csound and issue kill
		char logFile [ strlen ( obj->_spatialPak->name ) + strlen ( mixwaveResources->csoundPidlogFile() ) + 2 ];

		strncpy ( logFile, obj->_spatialPak->name, sizeof ( logFile ) );
		logFile [ sizeof ( logFile ) - 1 ] = 0;
		strncat ( logFile, "_", sizeof ( logFile ) - strlen ( logFile ) - 1 );
		logFile [ sizeof ( logFile ) - 1 ] = 0;		
		strncat ( logFile, mixwaveResources->csoundPidlogFile(), sizeof ( logFile ) - strlen ( logFile ) - 1 );
		logFile [ sizeof ( logFile ) - 1 ] = 0;

		char command [ strlen ( mixwaveResources->killCmd() ) +
						strlen ( logFile ) + strlen ( mixwaveResources->pidofCmd() ) +
						strlen ( mixwaveResources->csoundCmd() ) 
						+ strlen ( mixwaveResources->shEvalPrefix() ) +
						strlen ( mixwaveResources->shEvalSuffix() ) + 2 ];
	
		strncpy ( command, mixwaveResources->killCmd(), sizeof ( command ) );
		command [ sizeof ( command ) - 1 ] = 0;
		strncat ( command, mixwaveResources->shEvalPrefix(), sizeof ( command ) - strlen ( command ) - 1 );
		command [ sizeof ( command ) - 1 ] = 0;
		strncat ( command, mixwaveResources->pidofCmd(), sizeof ( command ) - strlen ( command ) - 1 );
		command [ sizeof ( command ) - 1 ] = 0;
		strncat ( command, " ", sizeof ( command ) - strlen ( command ) - 1 );
		command [ sizeof ( command ) - 1 ] = 0;
		strncat ( command, mixwaveResources->csoundCmd(), sizeof ( command ) - strlen ( command ) - 1 );
		command [ sizeof ( command ) - 1 ] = 0;
		strncat ( command, mixwaveResources->shEvalSuffix(), sizeof ( command ) - strlen ( command ) - 1 );
		command [ sizeof ( command ) - 1 ] = 0;

		MixwaveUtils *utils = new MixwaveUtils;
		utils->sysCall ( command );
		delete utils;

		char msg[28];
		strcpy(msg, "Spatialisation interrupted." );
		messageDialogManager->post ( msg );
	}
	else		
		obj->csoundRun ( ( RunCsoundCmd * ) cmd );
}


void CreateSpatialTemplateCmd::csoundRun ( RunCsoundCmd *cmd )
{
	char msg[BUFSIZE];

	if ( strlen ( cmd->error() ) != 0 )
	{
		_error = 1;
		snprintf ( msg, sizeof ( msg ), "Error: %s", cmd->error() );
		msg [ sizeof ( msg ) - 1 ] = 0;
		messageDialogManager->post ( msg );
	}
	else
		rawToWav ( _spatialPak->name );	// pass base name (no suffix)
}


void CreateSpatialTemplateCmd::rawToWav ( char *baseName )
{
	// convert baseName.raw output by CSound to baseName.wav
	if ( strlen ( baseName ) == 0 )
		return;
	
	char rawFile [ strlen ( baseName ) + 14 ];
	char wavFile [ strlen ( baseName ) + 14 ];
	
	strcpy ( rawFile, baseName );
	strcat ( rawFile, "_spatial.raw" );
	strcpy ( wavFile, baseName );
	strcat ( wavFile, "_spatial.wav" );

	// get the csound run log file and check for error string
	char logfile [ strlen (_spatialPak->name ) + strlen ( mixwaveResources->csoundRunlogFile() ) + 2 ];
	strcpy ( logfile, _spatialPak->name );
	strcat ( logfile, "_" );
	strcat ( logfile, mixwaveResources->csoundRunlogFile() );
	logfile [ sizeof ( logfile ) - 1 ] = 0;

	char buf [BUFSIZE];

	FileAccess *in = new FileAccess ( logfile, "r" );
	char msg [ in->fileSize() + 50 ];

	while ( in->filePos() < in->fileSize() )
	{
		in->readF ( buf );
		strncat ( msg, buf, sizeof ( msg ) );
	}

	delete in;

	if ( strstr ( msg, mixwaveResources->csoundErrorString() ) == NULL )
	{
		// no error in cound run log, wait pending raw file availability
		unsigned int error = 1;

		while ( error )
		{
			FileAccess *src = new FileAccess ( rawFile, "r" );
			error = src->errNum();
			delete src;
		}
	}	// on csound error, catch rawToWav conversion failure
	else
	{
		// display csound run log file
		strcat ( msg, "\nCreate raw data output file failed, given up." );
		messageDialogManager->post ( msg );
		_error = 1;
		return;
	}

	RawToWavCmd *rtwCmd = new RawToWavCmd ( "rawToWav", TRUE, rawFile, wavFile );

	rtwCmd->setWaveDefaults ( mixwaveResources->samplesPerSec(),
					mixwaveResources->bitsPerSample(), 2 );

//	setBusyCursor();
	rtwCmd->execute ( &CreateSpatialTemplateCmd::rawToWavFinishedCallback, ( void * ) this );
}


void CreateSpatialTemplateCmd::rawToWavFinishedCallback ( BreakCmd *cmd, Boolean interrupted, void *clientData )
{
	RawToWavCmd *rtwObj = ( RawToWavCmd * ) cmd;
	CreateSpatialTemplateCmd *obj = ( CreateSpatialTemplateCmd * ) clientData;
	
	if ( interrupted )
		messageDialogManager->post ( "Raw to wave conversion interrupted!" );
	else
		obj->rawToWavFinished ( rtwObj );
	
//	obj->setNormalCursor();
	delete rtwObj;
}


void CreateSpatialTemplateCmd::rawToWavFinished ( RawToWavCmd *rtwObj )
{
	if ( rtwObj->error() == 0 )
	{
		char msg[BUFSIZE];

		if ( mixwaveResources->enableMessagePopups() == True )
		{
			snprintf ( msg, sizeof ( msg ), "%s\nSpatialisation done; \
written\n%s_spatial.wav", (*wavePak)[_waveNum-1].pWave->name(), _spatialPak->name );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}

		// remove .raw file from csound output
		char command [ strlen ( _spatialPak->name ) + strlen ( mixwaveResources->removeCmd() ) + 14 ];
		strcpy ( command, mixwaveResources->removeCmd() );
		strcat ( command, " " );
		strcat ( command, _spatialPak->name );
		strcat ( command, "_spatial.raw" );
		command [ sizeof ( command ) - 1 ] = 0;

		MixwaveUtils *utils = new MixwaveUtils;
		utils->sysCall ( command );
		delete utils;
	}
}


float * CreateSpatialTemplateCmd::createFloatDataArray ( char *filename, float *fltArray, long blockSize, long filePos )
{
	delete[] fltArray;	// delete original array pointer
	_currentSamples = ( int ) blockSize / sizeof ( float );
	fltArray = new float [ _currentSamples ];

	FileAccess *in = new FileAccess ( filename, "r+" );

	if ( in->errNum() == 0 )	// file exists or is created
	{
		in->numElements ( _currentSamples );
		in->elementSize ( sizeof ( float ) );
		in->filePos ( filePos );
		in->readF ( ( void * ) fltArray );
	}
	else
		printf ( "CreateSpatialTemplateCmd::createFloatDataArray\n\
Error: file %s doesn't exist\n", filename );

	delete in;
	return fltArray;
}


void CreateSpatialTemplateCmd::minmax ( long start, long end, long selStart, long selEnd )
{
	long min, max;
	
	min = start;

	if ( end < min )
		min = end;
		
	if ( selStart != selEnd )
	{
		if ( selStart < min )
			min = selStart;
	
		if ( selEnd < min )
			min = selEnd;
	}
		
	max = start;
	
	if ( end > max )
		max = end;

	if ( selStart != selEnd )
	{
		if ( selStart > max )
			max = selStart;

		if ( selEnd > max )
			max = selEnd;
	}

	_min = min;
	_max = max;
}


void CreateSpatialTemplateCmd::exit()
{
	if ( _shapeCmdRealised == True )
	{
		delete _shapeCmd;
		_shapeCmdRealised = False;
	}

	// close all dependent views other than waveData, template creation done
	if ( _menuRealised == True )	// else deleted in menuSelectionCallback
	{
		deleteMenuObjects();
		delete _menu;
		_menuRealised = False;
	}

	if ( _pathMenuRealised == True )
	{
		deletePathMenuObjects();
		delete _pathMenu;
		_pathMenuRealised = False;
	}

	if ( _paramRangeMenuRealised == True )
	{
		deleteParamsRangeMenuObjects();
		delete _paramRangeMenu;
		_paramRangeMenuRealised = False;
	}

	_viewRealised = False;
}


void CreateSpatialTemplateCmd::initError ( char *file, char *error )
{
	char msg[BUFSIZE<<1];

	snprintf ( msg, sizeof ( msg ), "%s\nError: %s\nSpatial template creation disabled.", file, error );
	msg [ sizeof ( msg ) - 1 ] = 0;
	messageDialogManager->post ( msg );
	_error = 1; 	// global error flag - halt on any error
}


void CreateSpatialTemplateCmd::deleteMenuObjects()
{
	// close/delete sub-menu views
	if ( _rvbViewRealised == True )
	{
		_spatialPak->rvbView->close();
		_rvbViewRealised = False;
	}	

	if ( _spatialPak->rvbView != NULL )
	{
		delete _spatialPak->rvbView;
		_spatialPak->rvbView = NULL;
	}

	if ( _rangeViewRealised == True )
	{
		_spatialPak->rangeView->close();
		_rangeViewRealised = False;
	}	

	if ( _spatialPak->rangeView != NULL )
	{
		delete _spatialPak->rangeView;
		_spatialPak->rangeView = NULL;
	}

	if ( _bwViewRealised == True )
	{
		_spatialPak->bwView->close();
		_bwViewRealised = False;
	}	

	if ( _spatialPak->bwView != NULL )
	{
		delete _spatialPak->bwView;
		_spatialPak->bwView = NULL;
	}

	if ( _sweepViewRealised == True )
	{
		_spatialPak->sweepView->close();
		_sweepViewRealised = False;
	}	

	if ( _spatialPak->sweepView != NULL )
	{
		delete _spatialPak->sweepView;
		_spatialPak->sweepView = NULL;
	}

	if ( _panViewRealised == True )
	{
		_spatialPak->panView->close();
		_panViewRealised = False;
	}	

	if ( _spatialPak->panView != NULL )
	{
		delete _spatialPak->panView;
		_spatialPak->panView = NULL;
	}

	if ( _dopplerViewRealised == True )
	{
		_spatialPak->dopplerView->close();
		_dopplerViewRealised = False;
	}	

	if ( _spatialPak->dopplerView != NULL )
	{
		delete _spatialPak->dopplerView;
		_spatialPak->dopplerView = NULL;
	}

	if ( _panTypeEditRealised == True )
	{
		_panTypeEdit->close();
		_panTypeEditRealised = False;
	}

	if ( _panTypeEdit != NULL )
	{
		delete _panTypeEdit;	
		_panTypeEdit = NULL;
	}

	if ( _filterTypeEditRealised == True )
	{
		_filterTypeEdit->close();
		_filterTypeEditRealised = False;
	}

	if ( _filterTypeEdit != NULL )
	{
		delete _filterTypeEdit;	
		_filterTypeEdit = NULL;
	}

	if ( _angularSepEditRealised == True )
	{
		_angularSepEdit->close();
		_angularSepEditRealised = False;
	}

	if ( _angularSepEdit != NULL )
	{
		delete _angularSepEdit;	
		_angularSepEdit = NULL;
	}

	if ( _signalScaleEditRealised == True )
	{
		_signalScaleEdit->close();
		_signalScaleEditRealised = False;
	}

	if ( _signalScaleEdit != NULL )
	{
		delete _signalScaleEdit;	
		_signalScaleEdit = NULL;
	}

	if ( _detuneEditRealised == True )
	{
		_detuneEdit->close();
		_detuneEditRealised = False;
	}

	if ( _detuneEdit != NULL )
	{
		delete _detuneEdit;	
		_detuneEdit = NULL;
	}

	if ( _phaserEditRealised == True )
	{
		_phaserEdit->close();
		_phaserEditRealised = False;
	}

	if ( _phaserEdit != NULL )
	{
		delete _phaserEdit;	
		_phaserEdit = NULL;
	}

	if ( _rvbTailEditRealised == True )
	{
		_rvbTailEdit->close();
		_rvbTailEditRealised = False;
	}

	if ( _rvbTailEdit != NULL )
	{
		delete _rvbTailEdit;	
		_rvbTailEdit = NULL;
	}

	if ( _cssCmd != NULL )
	{
		delete _cssCmd;
		_cssCmd = NULL;
	}
	
	if ( _runCsndCmd != NULL )
	{
		delete _runCsndCmd;
		_runCsndCmd = NULL;
	}
}


void CreateSpatialTemplateCmd::deletePathMenuObjects()
{
	if ( _rvbLViewRealised == True )
	{
		_spatialPak->rvbLView->close();
		_rvbLViewRealised = False;
	}	

	if ( _spatialPak->rvbLView != NULL )
	{
		delete _spatialPak->rvbLView;
		_spatialPak->rvbLView = NULL;
	}

	if ( _rvbRViewRealised == True )
	{
		_spatialPak->rvbRView->close();
		_rvbRViewRealised = False;
	}	

	if ( _spatialPak->rvbRView != NULL )
	{
		delete _spatialPak->rvbRView;
		_spatialPak->rvbRView = NULL;
	}

	if ( _panLViewRealised == True )
	{
		_spatialPak->panLView->close();
		_panLViewRealised = False;
	}	

	if ( _spatialPak->panLView != NULL )
	{
		delete _spatialPak->panLView;
		_spatialPak->panLView = NULL;
	}

	if ( _panRViewRealised == True )
	{
		_spatialPak->panRView->close();
		_panRViewRealised = False;
	}	

	if ( _spatialPak->panRView != NULL )
	{
		delete _spatialPak->panRView;
		_spatialPak->panRView = NULL;
	}

	if ( _bwLViewRealised == True )
	{
		_spatialPak->bwLView->close();
		_bwLViewRealised = False;
	}	

	if ( _spatialPak->bwLView != NULL )
	{
		delete _spatialPak->bwLView;
		_spatialPak->bwLView = NULL;
	}

	if ( _bwRViewRealised == True )
	{
		_spatialPak->bwRView->close();
		_bwRViewRealised = False;
	}	

	if ( _spatialPak->bwRView != NULL )
	{
		delete _spatialPak->bwRView;
		_spatialPak->bwRView = NULL;
	}

	if ( _sweepLViewRealised == True )
	{
		_spatialPak->sweepLView->close();
		_sweepLViewRealised = False;
	}	

	if ( _spatialPak->sweepLView != NULL )
	{
		delete _spatialPak->sweepLView;
		_spatialPak->sweepLView = NULL;
	}

	if ( _sweepRViewRealised == True )
	{
		_spatialPak->sweepRView->close();
		_sweepRViewRealised = False;
	}	

	if ( _spatialPak->sweepRView != NULL )
	{
		delete _spatialPak->sweepRView;
		_spatialPak->sweepRView = NULL;
	}

	if ( _ampLViewRealised == True )
	{
		_spatialPak->ampLView->close();
		_ampLViewRealised = False;
	}	

	if ( _spatialPak->ampLView != NULL )
	{
		delete _spatialPak->ampLView;
		_spatialPak->ampLView = NULL;
	}

	if ( _ampRViewRealised == True )
	{
		_spatialPak->ampRView->close();
		_ampRViewRealised = False;
	}	

	if ( _spatialPak->ampRView != NULL )
	{
		delete _spatialPak->ampRView;
		_spatialPak->ampRView = NULL;
	}

	if ( _panLTypeEditRealised == True )
	{
		_panLTypeEdit->close();
		_panLTypeEditRealised = False;
	}

	if ( _panLTypeEdit != NULL )
	{
		delete _panLTypeEdit;	
		_panLTypeEdit = NULL;
	}

	if ( _panRTypeEditRealised == True )
	{
		_panRTypeEdit->close();
		_panRTypeEditRealised = False;
	}

	if ( _panRTypeEdit != NULL )
	{
		delete _panRTypeEdit;	
		_panRTypeEdit = NULL;
	}

	if ( _delayLEditRealised == True )
	{
		_delayLEdit->close();
		_delayLEditRealised = False;
	}

	if ( _delayLEdit != NULL )
	{
		delete _delayLEdit;	
		_delayLEdit = NULL;
	}

	if ( _delayREditRealised == True )
	{
		_delayREdit->close();
		_delayREditRealised = False;
	}

	if ( _delayREdit != NULL )
	{
		delete _delayREdit;	
		_delayREdit = NULL;
	}

	if ( _filterLTypeEditRealised == True )
	{
		_filterLTypeEdit->close();
		_filterLTypeEditRealised = False;
	}	

	if ( _filterLTypeEdit != NULL )
	{
		delete _filterLTypeEdit;
		_filterLTypeEdit = NULL;
	}

	if ( _filterRTypeEditRealised == True )
	{
		_filterRTypeEdit->close();
		_filterRTypeEditRealised = False;
	}	

	if ( _filterRTypeEdit != NULL )
	{
		delete _filterRTypeEdit;
		_filterRTypeEdit = NULL;
	}
}


void CreateSpatialTemplateCmd::deleteParamsRangeMenuObjects()
{
	if ( _rvbRangeEditRealised == True )
	{
		_rvbRangeEdit->close();
		_rvbRangeEditRealised = False;
	}	

	if ( _rvbRangeEdit != NULL )
	{
		delete _rvbRangeEdit;
		_rvbRangeEdit = NULL;
	}

	if ( _distRangeEditRealised == True )
	{
		_distRangeEdit->close();
		_distRangeEditRealised = False;
	}	

	if ( _distRangeEdit != NULL )
	{
		delete _distRangeEdit;
		_distRangeEdit = NULL;
	}

	if ( _bwRangeEditRealised == True )
	{
		_bwRangeEdit->close();
		_bwRangeEditRealised = False;
	}	

	if ( _bwRangeEdit != NULL )
	{
		delete _bwRangeEdit;
		_bwRangeEdit = NULL;
	}

	if ( _sweepRangeEditRealised == True )
	{
		_sweepRangeEdit->close();
		_sweepRangeEditRealised = False;
	}	

	if ( _sweepRangeEdit != NULL )
	{
		delete _sweepRangeEdit;
		_sweepRangeEdit = NULL;
	}

	if ( _dopplerRangeEditRealised == True )
	{
		_dopplerRangeEdit->close();
		_dopplerRangeEditRealised = False;
	}	

	if ( _dopplerRangeEdit != NULL )
	{
		delete _dopplerRangeEdit;
		_dopplerRangeEdit = NULL;
	}

	if ( _rvblRangeEditRealised == True )
	{
		_rvblRangeEdit->close();
		_rvblRangeEditRealised = False;
	}	

	if ( _rvblRangeEdit != NULL )
	{
		delete _rvblRangeEdit;
		_rvblRangeEdit = NULL;
	}

	if ( _rvbrRangeEditRealised == True )
	{
		_rvbrRangeEdit->close();
		_rvbrRangeEditRealised = False;
	}	

	if ( _rvbrRangeEdit != NULL )
	{
		delete _rvbrRangeEdit;
		_rvbrRangeEdit = NULL;
	}

	if ( _bwlRangeEditRealised == True )
	{
		_bwlRangeEdit->close();
		_bwlRangeEditRealised = False;
	}	

	if ( _bwlRangeEdit != NULL )
	{
		delete _bwlRangeEdit;
		_bwlRangeEdit = NULL;
	}

	if ( _bwrRangeEditRealised == True )
	{
		_bwrRangeEdit->close();
		_bwrRangeEditRealised = False;
	}	

	if ( _bwrRangeEdit != NULL )
	{
		delete _bwrRangeEdit;
		_bwrRangeEdit = NULL;
	}

	if ( _sweeplRangeEditRealised == True )
	{
		_sweeplRangeEdit->close();
		_sweeplRangeEditRealised = False;
	}	

	if ( _sweeplRangeEdit != NULL )
	{
		delete _sweeplRangeEdit;
		_sweeplRangeEdit = NULL;
	}

	if ( _sweeprRangeEditRealised == True )
	{
		_sweeprRangeEdit->close();
		_sweeprRangeEditRealised = False;
	}	

	if ( _sweeprRangeEdit != NULL )
	{
		delete _sweeprRangeEdit;
		_sweeprRangeEdit = NULL;
	}
	
	if ( _amplRangeEditRealised == True )
	{
		_amplRangeEdit->close();
		_amplRangeEditRealised = False;
	}	

	if ( _amplRangeEdit != NULL )
	{
		delete _amplRangeEdit;
		_amplRangeEdit = NULL;
	}

	if ( _amprRangeEditRealised == True )
	{
		_amprRangeEdit->close();
		_amprRangeEditRealised = False;
	}	

	if ( _amprRangeEdit != NULL )
	{
		delete _amprRangeEdit;
		_amprRangeEdit = NULL;
	}
}
